/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtButtons.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 19.01.2009
 *******************************************************************/

/*-------- include files:
-------------------------------------------------------------------*/
#include "QBtButtons.h"
#include "QBtEventsController.h"
#include <QPushButton>
#include <QHBoxLayout>
#include <QtDebug>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtButtons::F1_Key  = QT_TR_NOOP( "F1" );
const char* const QBtButtons::F2_Key  = QT_TR_NOOP( "F2" );
const char* const QBtButtons::F3_Key  = QT_TR_NOOP( "F3" );
const char* const QBtButtons::F4_Key  = QT_TR_NOOP( "F4" );
const char* const QBtButtons::F5_Key  = QT_TR_NOOP( "F5" );
const char* const QBtButtons::F6_Key  = QT_TR_NOOP( "F6" );
const char* const QBtButtons::F7_Key  = QT_TR_NOOP( "F7" );
const char* const QBtButtons::F8_Key  = QT_TR_NOOP( "F8" );
const char* const QBtButtons::F9_Key  = QT_TR_NOOP( "F9" );
const char* const QBtButtons::F10_Key = QT_TR_NOOP( "F10" );

const char* const QBtButtons::F1_Text  = QT_TR_NOOP( "Help" );
const char* const QBtButtons::F2_Text  = QT_TR_NOOP( "Access" );
const char* const QBtButtons::F3_Text  = QT_TR_NOOP( "View" );
const char* const QBtButtons::F4_Text  = QT_TR_NOOP( "Info" );
const char* const QBtButtons::F5_Text  = QT_TR_NOOP( "Proccess" );
const char* const QBtButtons::F6_Text  = QT_TR_NOOP( "Rename" );
const char* const QBtButtons::F7_Text  = QT_TR_NOOP( "MkDir" );
const char* const QBtButtons::F8_Text  = QT_TR_NOOP( "Delete" );
const char* const QBtButtons::F10_Text = QT_TR_NOOP( "Quit" );

//*******************************************************************
// QBtButtons                                            CONSTRUCTOR
//*******************************************************************
QBtButtons::QBtButtons( QWidget* const in_parent )
: QWidget( in_parent )
, F1_    ( new QPushButton )
, F2_    ( new QPushButton )
, F3_    ( new QPushButton )
, F4_    ( new QPushButton )
, F5_    ( new QPushButton )
, F6_    ( new QPushButton )
, F7_    ( new QPushButton )
, F8_    ( new QPushButton )
, F9_    ( new QPushButton )
, F10_   ( new QPushButton )
{
   F2_->setEnabled( false );
   F6_->setEnabled( false );
   F9_->setEnabled( false );

   F1_->setFocusPolicy( Qt::NoFocus );
   F2_->setFocusPolicy( Qt::NoFocus );
   F3_->setFocusPolicy( Qt::NoFocus );
   F4_->setFocusPolicy( Qt::NoFocus );
   F5_->setFocusPolicy( Qt::NoFocus );
   F6_->setFocusPolicy( Qt::NoFocus );
   F7_->setFocusPolicy( Qt::NoFocus );
   F8_->setFocusPolicy( Qt::NoFocus );
   F9_->setFocusPolicy( Qt::NoFocus );
   F10_->setFocusPolicy( Qt::NoFocus );
   
   QHBoxLayout* const main_layout = new QHBoxLayout;
   main_layout->addWidget( F1_ );
   main_layout->addWidget( F2_ );
   main_layout->addWidget( F3_ );
   main_layout->addWidget( F4_ );
   main_layout->addWidget( F5_ );
   main_layout->addWidget( F6_ );
   main_layout->addWidget( F7_ );
   main_layout->addWidget( F8_ );
   main_layout->addWidget( F9_ );
   main_layout->addWidget( F10_ );
   main_layout->setSpacing( 6 );
   main_layout->setContentsMargins( 0, 0, 0, 0 );
   setLayout( main_layout );

   retranslate();

   connect( F1_ , SIGNAL( clicked() ), this, SLOT ( F1 () ) );
   connect( F3_ , SIGNAL( clicked() ), this, SLOT ( F3 () ) );
   connect( F4_ , SIGNAL( clicked() ), this, SLOT ( F4 () ) );
   connect( F5_ , SIGNAL( clicked() ), this, SLOT ( F5 () ) );
   connect( F7_ , SIGNAL( clicked() ), this, SLOT ( F7 () ) );
   connect( F8_ , SIGNAL( clicked() ), this, SLOT ( F8 () ) );
   connect( F10_, SIGNAL( clicked() ), this, SLOT ( F10() ) );
}
// end of QBtButtons

//*******************************************************************
// retranslate                                               PRIVATE
//*******************************************************************
void QBtButtons::retranslate()
{
   F1_ ->setText( tr( F1_Key  ) + " " + tr( F1_Text ) );
   //F2_ ->setText( tr( F2_Key  ) + " " + tr( F2_Text ) );
   F3_ ->setText( tr( F3_Key  ) + " " + tr( F3_Text ) );
   F4_ ->setText( tr( F4_Key  ) + " " + tr( F4_Text ) );
   F5_ ->setText( tr( F5_Key  ) + " " + tr( F5_Text ) );
   //F6_ ->setText( tr( F6_Key  ) + " " + tr( F6_Text ) );
   F7_ ->setText( tr( F7_Key  ) + " " + tr( F7_Text ) );
   F8_ ->setText( tr( F8_Key  ) + " " + tr( F8_Text ) );
   //F9_ ->setText( tr( F9_Key  ) );
   F10_->setText( tr( F10_Key ) + " " + tr( F10_Text ) );
}
// end of retranslate

//*******************************************************************
// F1                                                   PRIVATE slot
//*******************************************************************
void QBtButtons::F1()
{
   QBtEventsController::instance()->send_event( QBtEvent::F1 );
}
// end of F1

//*******************************************************************
// F3                                                   PRIVATE slot
//*******************************************************************
void QBtButtons::F3()
{
   QBtEventsController::instance()->send_event( QBtEvent::F3 );
}
// end of F3

//*******************************************************************
// F4                                                   PRIVATE slot
//*******************************************************************
void QBtButtons::F4()
{
   QBtEventsController::instance()->send_event( QBtEvent::F4 );
}
// end of F4

//*******************************************************************
// F5                                                   PRIVATE slot
//*******************************************************************
void QBtButtons::F5()
{
   QBtEventsController::instance()->send_event( QBtEvent::F5 );
}
// end of F5

//*******************************************************************
// F7                                                   PRIVATE slot
//*******************************************************************
void QBtButtons::F7()
{
   QBtEventsController::instance()->send_event( QBtEvent::F7 );
}
// end of F7

//*******************************************************************
// F8                                                   PRIVATE slot
//*******************************************************************
void QBtButtons::F8()
{
   QBtEventsController::instance()->send_event( QBtEvent::F8 );
}
// end of F8

//*******************************************************************
// F10                                                  PRIVATE slot
//*******************************************************************
void QBtButtons::F10()
{
   QBtEventsController::instance()->send_event( QBtEvent::F10 );
}
// end of F10
