/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtCmdWidget.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 8.01.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtShared.h"
#include "QBtCmdWidget.h"
#include "QBtEventsController.h"
#include <QTreeWidget>
#include <QPushButton>
#include <QRadioButton>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QDir>
#include <QFile>
#include <QLabel>
#include <QFileDialog>
#include <QTextStream>
#include <QMessageBox>
#include <QtDebug>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtCmdWidget::EnableEdit  = QT_TR_NOOP( "&enabled edit" );
const char* const QBtCmdWidget::DisableEdit = QT_TR_NOOP( "&disabled edit" );
const char* const QBtCmdWidget::Delete      = QT_TR_NOOP( "&Delete" );
const char* const QBtCmdWidget::Save        = QT_TR_NOOP( "&Save" );
const char* const QBtCmdWidget::SaveAs      = QT_TR_NOOP( "Save &As" );
const char* const QBtCmdWidget::Read        = QT_TR_NOOP( "&Read" );
const char* const QBtCmdWidget::CmdFileName = QT_TR_NOOP( "Commands file:" );
const char* const QBtCmdWidget::NoName      = QT_TR_NOOP( "none" );
const char* const QBtCmdWidget::Caption     = QT_TR_NOOP( "Commands" );
const char* const QBtCmdWidget::SaveError   = QT_TR_NOOP( "Error on saving the file:\n%1\n%2" );
const char* const QBtCmdWidget::ReadError   = QT_TR_NOOP( "Error on reading the file:\n%1\n%2" );
const char* const QBtCmdWidget::Filters     = QT_TR_NOOP( "Commands files (*.cmc);;All files (*)" );
const char* const QBtCmdWidget::IsEmpty     = QT_TR_NOOP( "The file is empty." );
const char* const QBtCmdWidget::IsModified  = QT_TR_NOOP( "The current file was modified.\nWould you like to save him befor read a new file?" );

const QString QBtCmdWidget::Ext = QString( ".cmc" );

//*******************************************************************
// QBtCmdWidget                                          CONSTRUCTOR
//*******************************************************************
QBtCmdWidget::QBtCmdWidget( QWidget* const in_parent )
: QWidget    ( in_parent )
, view_      ( new QTreeWidget )
//, enable_    ( new QRadioButton( tr( EnableEdit ) ) )
//, disable_   ( new QRadioButton( tr( DisableEdit ) ) )
, up_        ( new QPushButton( QIcon( ":/img/up.png" ), QString() ) )
, down_      ( new QPushButton( QIcon( ":/img/down.png" ), QString() ) )
, delete_    ( new QPushButton( QIcon( ":/img/trash.png" ), QString() ) )
, save_      ( new QPushButton( tr( Save   ) ) )
, save_as_   ( new QPushButton( tr( SaveAs ) ) )
, read_      ( new QPushButton( tr( Read   ) ) )
, file_name_ ( new QLabel( tr( NoName ) ) )
, fpath_     ( QString() )
{
   //view_->setEditTriggers( QTreeWidget::DoubleClicked );
   view_->setColumnCount( 2 );
   view_->setHeaderHidden( true );
   view_->setRootIsDecorated( false );

   QHBoxLayout* const btn_layout = new QHBoxLayout;
//   btn_layout->addWidget( enable_ );
//   btn_layout->addWidget( disable_ );
   btn_layout->addStretch();
   btn_layout->addWidget( new QLabel( tr( CmdFileName ) ) );
   btn_layout->addWidget( file_name_ );
   btn_layout->addStretch();
   btn_layout->addWidget( save_ );
   btn_layout->addWidget( save_as_ );
   btn_layout->addWidget( read_ );

   QVBoxLayout* updown_layout = new QVBoxLayout;
   updown_layout->addStretch( 20 );
   updown_layout->addWidget( delete_ );
   updown_layout->addStretch( 20 );
   updown_layout->addWidget( up_ );
   updown_layout->addWidget( down_ );
   updown_layout->addStretch( 50 );
   
   QGridLayout* const main_layout = new QGridLayout;
   main_layout->addLayout( btn_layout   , 0, 0 );
   main_layout->addWidget( view_        , 1, 0 );
   main_layout->addLayout( updown_layout, 1, 1 );
   setLayout( main_layout );

// connect( enable_ , SIGNAL( clicked( bool ) ), this, SLOT( enable_slot( bool ) ) );
// connect( disable_, SIGNAL( clicked( bool ) ), this, SLOT( disable_slot( bool ) ) );
   connect( delete_ , SIGNAL( clicked() ), this, SLOT( delete_clicked() ) );
   connect( up_     , SIGNAL( clicked() ), this, SLOT( up_clicked() ) );
   connect( down_   , SIGNAL( clicked() ), this, SLOT( down_clicked() ) );
   connect( save_   , SIGNAL( clicked() ), this, SLOT( save_slot() ) );
   connect( save_as_, SIGNAL( clicked() ), this, SLOT( save_as_slot() ) );
   connect( read_   , SIGNAL( clicked() ), this, SLOT( read_slot() ) );

// disable_->setChecked( true );
// disable_slot( true );

   setFocusPolicy( Qt::NoFocus );

   QBtEventsController* const evc = QBtEventsController::instance();
   evc->append( this, QBtEvent::SaveCommand );
   evc->append( this, QBtEvent::ParamsRequest );
   evc->append( this, QBtEvent::CommandsAndParamsRequest );
}
// end of QBtCmdWidget

//*******************************************************************
// ~QBtCmdWidget                                          DESTRUCTOR
//*******************************************************************
QBtCmdWidget::~QBtCmdWidget()
{
   QBtEventsController::instance()->remove( this );
}
// end of ~QBtCmdWidget

//*******************************************************************
// commands                                                   PUBLIC
//*******************************************************************
QStringList QBtCmdWidget::commands() const
{
   QStringList data = QStringList();
   const QTreeWidgetItem* const root = view_->invisibleRootItem();
   const int n = root->childCount();
   QString text;

   for( int i = 0; i < n; ++i ) {
      const QTreeWidgetItem* const item = root->child( i );
      text = item->text( 0 );
      text += '#';
      text += item->text( 1 );
      data.push_back( text );
   }
   return data;
}
// end of commands

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtCmdWidget::customEvent( QEvent* const in_event )
{
   QBtEvent* const event = dynamic_cast<QBtEvent*>( in_event );
   const int type = static_cast<int>( event->type() );

   switch( type ) {
      case QBtEvent::SaveCommand:
         save_command( event->data( 0 ).toString(), event->data( 1 ).toString() );
         break;
      case QBtEvent::ParamsRequest:
         send_params( event->data( 0 ).toString() );
         break;
      case QBtEvent::CommandsAndParamsRequest:
         QBtEventsController::instance()->send_event( QBtEvent::CommandsAndParamsAnswer, commands() );
         break;
   }
}
// end of customEvent

//*******************************************************************
// send_params                                               PRIVATE
//*******************************************************************
void QBtCmdWidget::send_params( const QString& command )
{
   const QTreeWidgetItem* const item = search_item_for_command( command );
   const QString txt = item ? item->text( 1 ) : QString();
   QBtEventsController::instance()->send_event( QBtEvent::ParamsAnswer, txt );
}
// end of send_params

//*******************************************************************
// delete_clicked                                       PRIVATE slot
//-------------------------------------------------------------------
// Uzytkownik uzyl przycisku z koszykiem (prawa strona panelu).
// Usuwamy z listy aktualna pozycje.
//*******************************************************************
void QBtCmdWidget::delete_clicked()
{
   QTreeWidgetItem* const root = view_->invisibleRootItem();
   
   QTreeWidgetItem* const current_item = view_->currentItem();
   if( current_item ) {
      root->removeChild( current_item );
      delete current_item;
   }
}
// end of delete_clicked

//*******************************************************************
// up_clicked                                           PRIVATE slot
//*******************************************************************
void QBtCmdWidget::up_clicked()
{
   QTreeWidgetItem* const root = view_->invisibleRootItem();
   const int n = root->childCount();

   if( n ) {
      QTreeWidgetItem* const current_item = view_->currentItem();
      if( current_item ) {
         const int index = root->indexOfChild( current_item );
         if( index ) {
            root->removeChild( current_item );
            root->insertChild( index - 1, current_item );
            view_->setCurrentItem( current_item );
         }
      }
   }
}
// end of up_clicked

//*******************************************************************
// down_clicked                                         PRIVATE slot
//*******************************************************************
void QBtCmdWidget::down_clicked()
{
   QTreeWidgetItem* const root = view_->invisibleRootItem();
   const int n = root->childCount();

   if( n ) {
      QTreeWidgetItem* const current_item = view_->currentItem();
      if( current_item ) {
         const int index = root->indexOfChild( current_item );
         if( index < ( n - 1 ) ) {
            root->removeChild( current_item );
            root->insertChild( index + 1, current_item );
            view_->setCurrentItem( current_item );
         }
      }
   }
}
// end of down_clicked

//*******************************************************************
// save_slot                                            PRIVATE slot
//*******************************************************************
void QBtCmdWidget::save_slot()
{
   fpath_.isEmpty() ? save_as() : save( fpath_ );
}
// end of save_slot

//*******************************************************************
// save_as_slot                                         PRIVATE slot
//*******************************************************************
void QBtCmdWidget::save_as_slot()
{
   save_as();
}
// end of save_as_slot

//*******************************************************************
// read_slot                                            PRIVATE slot
//*******************************************************************
void QBtCmdWidget::read_slot()
{
   read();
}
// end of read_slot

//*******************************************************************
// save_command                                              PRIVATE
//*******************************************************************
void QBtCmdWidget::save_command( const QString command, const QString params )
{
   QTreeWidgetItem* const root = view_->invisibleRootItem();
   QTreeWidgetItem* this_item = search_item_for_command( command );
   
   if( this_item ) {
      if( params.isEmpty() ) root->removeChild( this_item );
      else                   this_item->setText( 1, params );
   }
   else {
      this_item = new QTreeWidgetItem;
      this_item->setText( 0, command );
      this_item->setText( 1, params );
      root->addChild( this_item );
      view_->resizeColumnToContents( 0 );
   }
}
// end of save_command

//*******************************************************************
// search_item_for_command                                   PRIVATE
//*******************************************************************
QTreeWidgetItem* QBtCmdWidget::search_item_for_command( const QString& command )
{
   const QTreeWidgetItem* const root = view_->invisibleRootItem();
   const int n = root->childCount();

   for( int i = 0; i < n; ++i ) {
      QTreeWidgetItem* const this_item = root->child( i );
      if( command == this_item->text( 0 ) ) {
         return this_item;
      }
   }
   return 0;
}
// end of search_item_for_command


//###################################################################
//#                                                                 #
//#              O P E R A C J E   D Y S K O W E                    #
//#                                                                 #
//###################################################################


//*******************************************************************
// save_as                                                   PRIVATE
//*******************************************************************
bool QBtCmdWidget::save_as()
{
  const QTreeWidgetItem* const root = view_->invisibleRootItem();
   const int n = root->childCount();

   if( 0 == n ) {
      QMessageBox::information( this, tr( Caption ), tr( IsEmpty ) );
      return false;     
   }
   
   QString nfpath = QFileDialog::getSaveFileName( this, QString(), QDir::homePath(), tr( Filters ) );
   
   if( nfpath.isEmpty() ) return false;
   if( !nfpath.endsWith( Ext ) ) nfpath += Ext;

   return save( nfpath );
 }
// end of save_as

//*******************************************************************
// save                                                      PRIVATE
//-------------------------------------------------------------------
// Zapisywanie komend do wskazanego pliku.
// Jesli plik o podanej nazwie juz istnieje jego wczesniejsza
// zawartosc zostanie skasowana.
//*******************************************************************
bool QBtCmdWidget::save( const QString& fpath )
{
   const QTreeWidgetItem* const root = view_->invisibleRootItem();
   const int n = root->childCount();

   if( 0 == n ) {
      QMessageBox::information( this, tr( Caption ), tr( IsEmpty ) );
      return false;     
   }

   QFile file( fpath );
   if( file.open( QIODevice::Truncate | QIODevice::WriteOnly ) ) {
      QTextStream out( &file );
      //------- zapis danych do pliku -------
      QBtShared::set_cursor( Qt::WaitCursor );
      for( int i = 0; i < n; ++i ) {
         const QTreeWidgetItem* const this_item = root->child( i );
         out << this_item->text( 0 );
         out << '#';
         out << this_item->text( 1 );
         out << '\n';
      }    
      QBtShared::restore_cursor();
      
      if( QFile::NoError == file.error() ) {
         file.close();
         file_name_->setText( fpath_ = fpath );
         return true;
      }
      
      file.close();
      QMessageBox::warning
         ( this,
           tr( Caption ),
           tr( SaveError ).arg( fpath ).arg( file.errorString() ) );
   }
   return false;
}
// end of save

//*******************************************************************
// read                                                      PRIVATE
//*******************************************************************
bool QBtCmdWidget::read()
{
   const QString fpath = QFileDialog::getOpenFileName( this, QString(), QDir::homePath(), tr( Filters ) );
   if( fpath.isEmpty() ) return false;

   QFile file( fpath );
   if( file.open( QIODevice::ReadOnly ) ) {
      view_->clear();
      QTextStream in( &file );
      //------- odczyt pliku ------------------------------
      QBtShared::set_cursor( Qt::WaitCursor );
      QString text, command, params;
      QTreeWidgetItem* const root = view_->invisibleRootItem();
      do {
         text = in.readLine();
         if( !text.isEmpty()) {
            const int idx = text.indexOf( '#' );
            if( idx != 1 ) {
               command = text.left( idx );
               params  = text.mid( idx + 1 );
               params.replace( '#', ';' );
               QTreeWidgetItem* const new_item = new QTreeWidgetItem;
               new_item->setText( 0, command );
               new_item->setText( 1, params );
               root->addChild( new_item );
            }
         }
      } while( !text.isNull());
      view_->resizeColumnToContents( 0 );
      QBtShared::restore_cursor();
      //.........................................
      
      if( QFile::NoError == file.error() ) {
         file.close();
         file_name_->setText( fpath_ = fpath );
         return true;
      }

      file.close();      
      QMessageBox::critical
         ( this,
           tr( Caption ),
           tr( ReadError ).arg( fpath ).arg( file.errorString() ) );
   }
   return false;
}
// end of read
