#ifndef INCLUDED_QBtCmdWidget_h
#define INCLUDED_QBtCmdWidget_h
/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtCmdWidget.h
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 8.01.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <QWidget>

/*------- forward declarations:
-------------------------------------------------------------------*/
class QTreeWidget;
class QPushButton;
class QRadioButton;
class QLabel;
class QTreeWidgetItem;

/*------- class declaration:
-------------------------------------------------------------------*/
class QBtCmdWidget : public QWidget
{
   Q_OBJECT

//******* CONSTRUCTION / DESTRUCTION *******
public:
   QBtCmdWidget( QWidget* = 0 );
   ~QBtCmdWidget();
private:
   QBtCmdWidget( const QBtCmdWidget& );
   QBtCmdWidget& operator=( const QBtCmdWidget& );

//******* CONSTANTS *******
private:
   static const char* const EnableEdit;
   static const char* const DisableEdit;
   static const char* const Delete;
   static const char* const Save;
   static const char* const SaveAs;
   static const char* const Read;
   static const char* const CmdFileName;
   static const char* const NoName;
   static const char* const Caption;
   static const char* const SaveError;
   static const char* const ReadError;
   static const char* const Filters;
   static const char* const IsEmpty;
   static const char* const IsModified;
   
   static const QString Ext;

//******* MEMBERS *******
private:
   QTreeWidget * const view_;
// QRadioButton* const enable_;
// QRadioButton* const disable_;
   QPushButton * const up_;
   QPushButton * const down_;
   QPushButton * const delete_;
   QPushButton * const save_;
   QPushButton * const save_as_;
   QPushButton * const read_;
   QLabel      * const file_name_;
   QString             fpath_;

//******* METHODS *******
public:
   QStringList commands        () const;
private slots:
// void       enable_slot      ( bool );
// void       disable_slot     ( bool );
   void       delete_clicked   ();
   void       up_clicked       ();
   void       down_clicked     ();
   void       save_slot        ();
   void       save_as_slot     ();
   void       read_slot        ();
private:
   void customEvent ( QEvent* );
   void send_params( const QString& );
   QTreeWidgetItem* search_item_for_command( const QString& );
   bool save   ( const QString& );
   bool save_as();
   bool read   ();
   void change_format( const QString );
   void save_command( const QString, const QString );
};

#endif // INCLUDED_QBtCmdWidget_h
