#ifndef INCLUDED_QBtConfigAnnotate_h
#define INCLUDED_QBtConfigAnnotate_h
/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtConfigAnnotate.h
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 25.03.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtConfigBaseDialog.h"

/*------- forward declarations:
-------------------------------------------------------------------*/
class QLineEdit;
class QPushButton;
class QSpinBox;
class QBtColorDemo;
class QRadioButton;

/*------- class declaration:
-------------------------------------------------------------------*/
class QBtConfigAnnotate : public QBtConfigBaseDialog
{
   Q_OBJECT

//------- CONSTRUCTION / DESTRUCTION -------
public:
   QBtConfigAnnotate( QWidget*, const QString& );
   ~QBtConfigAnnotate();
private:
   QBtConfigAnnotate( const QBtConfigAnnotate& );
   QBtConfigAnnotate& operator=( const QBtConfigAnnotate& );

//------- CONSTANTS -------
private:
   static const char* const Caption;
   static const char* const Picture;
   static const char* const Text;
   static const char* const Background;
   static const char* const Transparent;
   static const char* const XMargin;
   static const char* const YMargin;

//------- MEMBERS -------
private:
   QRadioButton* const picture_radio_btn_;
   QLineEdit   * const picture_path_edit_;
   QPushButton * const picture_path_btn_;
   QRadioButton* const text_radio_btn_;
   QLineEdit   * const text_edit_;
   QBtColorDemo* const color_demo_;
   QPushButton * const color_btn_;
   QSpinBox    * const trans_spin_box_;
   QSpinBox    * const x_margin_spin_box_;
   QSpinBox    * const y_margin_spin_box_;

//------- METHODS -------
private:
   QString get_command() const;
   void showEvent( QShowEvent* );
   void customEvent( QEvent* );
   void update_values( const QString& );
private slots:
   void try_clicked();
   void accept_clicked();
   void picture_radio_toggled( bool );
   void picture_path_clicked();
   void text_radio_toggled( bool );
   void color_clicked();
};

#endif // INCLUDED_QBtConfigAnnotate_h
