#ifndef INCLUDED_QBtConfigBaseDialog_h
#define INCLUDED_QBtConfigBaseDialog_h
/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtConfigBaseDialog.h
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 13.02.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtMagic.h"
#include <QDialog>

/*------- forward declarations:
-------------------------------------------------------------------*/
class QBtPicture;
class QPushButton;
class QGridLayout;
class QHBoxLayout;
class QVBoxLayout;

/*------- class declaration:
-------------------------------------------------------------------*/
class QBtConfigBaseDialog : public QDialog
{
   Q_OBJECT

//------- CONSTRUCTION / DESTRUCTION -------
public:
   QBtConfigBaseDialog( QWidget*, const QString& );
private:
   QBtConfigBaseDialog( const QBtConfigBaseDialog& );
   QBtConfigBaseDialog& operator=( const QBtConfigBaseDialog& );

//------- CONSTANTS -------
private:
   static const char* const Try;
   static const char* const TryAll;
   static const char* const Reset;
   static const char* const Cancel;
   static const char* const Accept;
   static const char* const Error;
   static const char* const Message;
   
//------- MEMBERS -------
protected:
   QBtPicture * const pictureA_;
   QBtPicture * const pictureB_;
   QPushButton* const try_;
   QPushButton* const accept_;
   QHBoxLayout* const user_layout_;
   QBtMagic           magic_;
   QImage             src_qimage_;
   QImage             dst_qimage_;
private:
   QPushButton* const try_all_;
   QPushButton* const reset_;
   QPushButton* const cancel_;
   QString            fpath_;
   bool               first_time_;
    
//------- METHODS -------
protected:
   void showEvent   ( QShowEvent*   );
   void resizeEvent ( QResizeEvent* );
private:
   void adjust      ();
protected:
   void resize_image( bool = true);
   void try_all ( const QStringList&, const QString&, const QString& );
   virtual QString get_command () const = 0;
private slots:
   void reset ();
   void try_all_clicked ();
};

 
#endif // INCLUDED_QBtConfigBaseDialog_h
