/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtConfigBorder.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 14.04.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtConfigBorder.h"
#include "QBtColorDemo.h"
#include "QBtEventsController.h"
#include "QBtShared.h"
#include "Parsery.h"
#include <QLabel>
#include <QSpinBox>
#include <QCheckBox>
#include <QPushButton>
#include <QGridLayout>
#include <QColorDialog>
//#include <QtDebug>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtConfigBorder::Caption      = QT_TR_NOOP( "Border" );
const char* const QBtConfigBorder::Horizontal   = QT_TR_NOOP( "&horizontal" );
const char* const QBtConfigBorder::Vertical     = QT_TR_NOOP( "&vertical:" );
const char* const QBtConfigBorder::Top          = QT_TR_NOOP( "&top" );
const char* const QBtConfigBorder::Bottom       = QT_TR_NOOP( "&bottom" );
const char* const QBtConfigBorder::Left         = QT_TR_NOOP( "&left" );
const char* const QBtConfigBorder::Right        = QT_TR_NOOP( "&right" );
const char* const QBtConfigBorder::Width        = QT_TR_NOOP( "width:" );
const char* const QBtConfigBorder::Transparency = QT_TR_NOOP( "transparency:" );
const char* const QBtConfigBorder::Order        = QT_TR_NOOP( "order:" );
const char* const QBtConfigBorder::Color        = QT_TR_NOOP( "color:" );


//*******************************************************************
// QBtConfigBorder                                    CONSTRUCTOR
//*******************************************************************
QBtConfigBorder::QBtConfigBorder( QWidget* const parent, const QString& fpath )
: QBtConfigBaseDialog( parent, fpath )
, horizontal_      ( new QCheckBox( tr( Horizontal ) ) )
, top_             ( new QCheckBox( tr( Top        ) ) )
, bottom_          ( new QCheckBox( tr( Bottom     ) ) )
, vertical_        ( new QCheckBox( tr( Vertical   ) ) )
, left_            ( new QCheckBox( tr( Left       ) ) )
, right_           ( new QCheckBox( tr( Right      ) ) )
//.....................................
, top_color_       ( new QBtColorDemo )
, bottom_color_    ( new QBtColorDemo )
, left_color_      ( new QBtColorDemo )
, right_color_     ( new QBtColorDemo )
//.....................................
, top_color_btn_   ( new QPushButton( QIcon( ":/img/colors.png" ), QString() ) )
, bottom_color_btn_( new QPushButton( QIcon( ":/img/colors.png" ), QString() ) )
, left_color_btn_  ( new QPushButton( QIcon( ":/img/colors.png" ), QString() ) )
, right_color_btn_ ( new QPushButton( QIcon( ":/img/colors.png" ), QString() ) )
//.....................................
, top_width_       ( new QSpinBox )
, bottom_width_    ( new QSpinBox )
, left_width_      ( new QSpinBox )
, right_width_     ( new QSpinBox )
//.....................................
, top_trans_       ( new QSpinBox )
, bottom_trans_    ( new QSpinBox )
, left_trans_      ( new QSpinBox )
, right_trans_     ( new QSpinBox )
//.....................................
, top_order_       ( new QSpinBox )
, bottom_order_    ( new QSpinBox )
, left_order_      ( new QSpinBox )
, right_order_     ( new QSpinBox )
//.....................................
, hcolor_          ( new QLabel( tr( Color        ) ) )
, vcolor_          ( new QLabel( tr( Color        ) ) )
, hwidth_          ( new QLabel( tr( Width        ) ) )
, vwidth_          ( new QLabel( tr( Width        ) ) )
, htrans_          ( new QLabel( tr( Transparency ) ) )
, vtrans_          ( new QLabel( tr( Transparency ) ) )
, horder_          ( new QLabel( tr( Order        ) ) )
, vorder_          ( new QLabel( tr( Order        ) ) )
{
   setWindowTitle( tr( Caption ) );

   QGridLayout* const hgrid = new QGridLayout;
   hgrid->addWidget( horizontal_      , 0,  0, 1, 2 );
   hgrid->addWidget( hcolor_          , 0,  3, 1, 2 );
   hgrid->addWidget( hwidth_          , 0,  6 );
   hgrid->addWidget( htrans_          , 0,  8 );
   hgrid->addWidget( horder_          , 0, 10 );
   hgrid->addWidget( top_             , 1,  1 );
   hgrid->addWidget( top_color_       , 1,  3 );
   hgrid->addWidget( top_color_btn_   , 1,  4 );
   hgrid->addWidget( top_width_       , 1,  6 );
   hgrid->addWidget( top_trans_       , 1,  8 );
   hgrid->addWidget( top_order_       , 1, 10 );
   hgrid->addWidget( bottom_          , 2,  1 );
   hgrid->addWidget( bottom_color_    , 2,  3 );
   hgrid->addWidget( bottom_color_btn_, 2,  4 );
   hgrid->addWidget( bottom_width_    , 2,  6 );
   hgrid->addWidget( bottom_trans_    , 2,  8 );
   hgrid->addWidget( bottom_order_    , 2, 10 );
   hgrid->setColumnMinimumWidth(  0, 10 );
   hgrid->setColumnMinimumWidth(  2, 10 );
   hgrid->setColumnMinimumWidth(  5, 10 );
   hgrid->setColumnMinimumWidth(  7, 10 );
   hgrid->setColumnMinimumWidth(  9, 10 );
   hgrid->setRowMinimumHeight( 10, 20 );

   QGridLayout* const vgrid = new QGridLayout;
   vgrid->addWidget( vertical_       , 0,  0, 1, 2 );
   vgrid->addWidget( vcolor_         , 0,  3, 1, 2 );
   vgrid->addWidget( vwidth_         , 0,  6 );
   vgrid->addWidget( vtrans_         , 0,  8 );
   vgrid->addWidget( vorder_         , 0, 10 );
   vgrid->addWidget( right_          , 1,  1 );
   vgrid->addWidget( right_color_    , 1,  3 );
   vgrid->addWidget( right_color_btn_, 1,  4 );
   vgrid->addWidget( right_width_    , 1,  6 );
   vgrid->addWidget( right_trans_    , 1,  8 );
   vgrid->addWidget( right_order_    , 1, 10 );  
   vgrid->addWidget( left_           , 2,  1 );
   vgrid->addWidget( left_color_     , 2,  3 );
   vgrid->addWidget( left_color_btn_ , 2,  4 );
   vgrid->addWidget( left_width_     , 2,  6 );
   vgrid->addWidget( left_trans_     , 2,  8 );
   vgrid->addWidget( left_order_     , 2, 10 );
   vgrid->setColumnMinimumWidth( 0, 10 );
   vgrid->setColumnMinimumWidth( 2, 10 );
   vgrid->setColumnMinimumWidth( 5, 10 );
   vgrid->setColumnMinimumWidth( 7, 10 );
   vgrid->setColumnMinimumWidth( 9, 10 );
   vgrid->setRowMinimumHeight( 10, 20 );
   
   user_layout_->addLayout( hgrid );
   user_layout_->addStretch();
   user_layout_->addLayout( vgrid );
   user_layout_->addStretch();
   
   horizontal_->setCheckState( Qt::Checked );
   top_->setCheckState( Qt::Checked );
   bottom_->setCheckState( Qt::Checked );
   vertical_->setCheckState( Qt::Checked );
   left_->setCheckState( Qt::Checked );
   right_->setCheckState( Qt::Checked );

   top_color_->setMinimumWidth( 50 );
   top_color_->setMaximumWidth( 60 );
   bottom_color_->setMinimumWidth( 50 );
   bottom_color_->setMaximumWidth( 60 );
   left_color_->setMinimumWidth( 50 );
   left_color_->setMaximumWidth( 60 );
   right_color_->setMinimumWidth( 50 );
   right_color_->setMaximumWidth( 60 );
   
   top_width_->setRange( 0, 50 );
   bottom_width_->setRange( 0, 50 );
   left_width_->setRange( 0, 50 );
   right_width_->setRange( 0, 50 );
   top_width_->setSuffix( "px" );
   bottom_width_->setSuffix( "px" );
   left_width_->setSuffix( "px" );
   right_width_->setSuffix( "px" );

   top_trans_->setRange( 0, 100 );
   bottom_trans_->setRange( 0, 100 );
   left_trans_->setRange( 0, 100 );
   right_trans_->setRange( 0, 100 );
   top_trans_->setSuffix( "%" );
   bottom_trans_->setSuffix( "%" );
   left_trans_->setSuffix( "%" );
   right_trans_->setSuffix( "%" );

   top_order_->setRange( 1, 4 );
   bottom_order_->setRange( 1, 4 );
   left_order_->setRange( 1, 4 );
   right_order_->setRange( 1, 4 );
   top_order_->setValue( 1 );
   right_order_->setValue( 2 );
   bottom_order_->setValue( 3 );
   left_order_->setValue( 4 );

   connect( try_   , SIGNAL( clicked() ), this, SLOT( try_clicked   () ) );
   connect( accept_, SIGNAL( clicked() ), this, SLOT( accept_clicked() ) );
   //......................................................
   connect( top_color_btn_   , SIGNAL( clicked() ), this, SLOT( top_color_clicked() ) );
   connect( bottom_color_btn_, SIGNAL( clicked() ), this, SLOT( bottom_color_clicked() ) );
   connect( left_color_btn_  , SIGNAL( clicked() ), this, SLOT( left_color_clicked() ) );
   connect( right_color_btn_ , SIGNAL( clicked() ), this, SLOT( right_color_clicked() ) );
   //......................................................
   connect( horizontal_, SIGNAL( stateChanged(int) ), this, SLOT( horizontal_changed(int) ) );
   connect( top_       , SIGNAL( stateChanged(int) ), this, SLOT( top_changed(int) ) );
   connect( bottom_    , SIGNAL( stateChanged(int) ), this, SLOT( bottom_changed(int) ) );
   connect( vertical_  , SIGNAL( stateChanged(int) ), this, SLOT( vertical_changed(int) ) );
   connect( left_      , SIGNAL( stateChanged(int) ), this, SLOT( left_changed(int) ) );
   connect( right_     , SIGNAL( stateChanged(int) ), this, SLOT( right_changed(int) ) );
   
   QBtEventsController* const evc = QBtEventsController::instance();
   evc->append( this, QBtEvent::ParamsAnswer );
   evc->append( this, QBtEvent::CommandsAndParamsAnswer );
}
// end of QBtConfigBorder

//*******************************************************************
// ~QBtConfigBorder                                    DESTRUCTOR
//*******************************************************************
QBtConfigBorder::~QBtConfigBorder()
{
   QBtEventsController::instance()->remove( this );
}
// end of ~QBtConfigBorder

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void QBtConfigBorder::showEvent( QShowEvent* const e )
{
   QBtConfigBaseDialog::showEvent( e );
   QBtEventsController::instance()->send_event( QBtEvent::ParamsRequest, "border" );
}
// end of showEvent

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtConfigBorder::customEvent( QEvent* const e )
{
   QBtEvent* const event = dynamic_cast<QBtEvent*>( e );
   const int type = static_cast<int>( event->type() );

   switch( type ) {
      case QBtEvent::ParamsAnswer:
         update_values( event->data( 0 ).toString() );
         break;
      case QBtEvent::CommandsAndParamsAnswer:
         try_all( event->data( 0 ).toStringList(), "border", get_command() );
   }
}
// end of customEvent

//*******************************************************************
// get_command                                     PRIVATE inherited
//*******************************************************************
QString QBtConfigBorder::get_command() const
{
   CBtBorderParser parser;
   
   if( horizontal_->checkState() == Qt::Checked ) {
      if( top_->checkState() == Qt::Checked ) {
         parser.top().color       ( top_color_->background() );
         parser.top().width       ( top_width_->value() );
         parser.top().transparency( top_trans_->value() );
         parser.top().order       ( top_order_->value() );
      }
      if( bottom_->checkState() == Qt::Checked ) {
         parser.bottom().color       ( bottom_color_->background() );
         parser.bottom().width       ( bottom_width_->value() );
         parser.bottom().transparency( bottom_trans_->value() );
         parser.bottom().order       ( bottom_order_->value() );
      }
   }
   if( vertical_->checkState() == Qt::Checked ) {
      if( left_->checkState() == Qt::Checked ) {
         parser.left().color       ( left_color_->background() );
         parser.left().width       ( left_width_->value() );
         parser.left().transparency( left_trans_->value() );
         parser.left().order       ( left_order_->value() );
      }
      if( right_->checkState() == Qt::Checked ) {
         parser.right().color       ( right_color_->background() );
         parser.right().width       ( right_width_->value() );
         parser.right().transparency( right_trans_->value() );
         parser.right().order       ( right_order_->value() );
      }
   }

   return parser.command();
}
// end of get_command

//*******************************************************************
// update_values                                             PRIVATE
//*******************************************************************
void QBtConfigBorder::update_values( const QString& params )
{
   if( params.isEmpty() ) return;
   
   CBtBorderParser parser( params );
   
   top_color_->background( parser.top().qt_color() );
   top_width_->setValue  ( parser.top().width() );
   top_trans_->setValue  ( parser.top().transparency() );
   top_order_->setValue  ( parser.top().order() );
   //................................................................
   bottom_color_->background( parser.bottom().qt_color() );
   bottom_width_->setValue  ( parser.bottom().width() );
   bottom_trans_->setValue  ( parser.bottom().transparency() );
   bottom_order_->setValue  ( parser.bottom().order() );
   //................................................................
   left_color_->background( parser.left().qt_color() );
   left_width_->setValue  ( parser.left().width() );
   left_trans_->setValue  ( parser.left().transparency() );
   left_order_->setValue  ( parser.left().order() );
   //................................................................
   right_color_->background( parser.right().qt_color() );
   right_width_->setValue  ( parser.right().width() );
   right_trans_->setValue  ( parser.right().transparency() );
   right_order_->setValue  ( parser.right().order() );
}
// end of update_values

//*******************************************************************
// try_clicked                                          PRIVATE slot
//*******************************************************************
void QBtConfigBorder::try_clicked()
{
   bool error = false;
   
   QBtShared::disable_updates( this );
   try {
      QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
      QBtMagic tmp = magic_;
      tmp.border( get_command() );
      dst_qimage_ = tmp.qimage();
      resize_image( false );
   }
   catch( Magick::Warning& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::OutputMessage, QString( e.what() ) );
      error = true;
   }
   catch( Magick::Error& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( e.what() ) );
      error = true;
   }
   catch(...) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( "???" ) );
      error = true;
   }
   QBtShared::enable_updates( this );

   if( error ) reject();
}
// end of try_clicked

//*******************************************************************
// accept_clicked                                       PRIVATE slot
//*******************************************************************
void QBtConfigBorder::accept_clicked()
{
   QBtEventsController::instance()->send_event( QBtEvent::SaveCommand, "border", get_command() );
   accept(); 
}
// end of accept_clicked

//*******************************************************************
// horizontal_changed                                   PRIVATE slot
//*******************************************************************
void QBtConfigBorder::horizontal_changed( const int state )
{
   const bool is_enabled = ( Qt::Checked == state );
   top_->setEnabled( is_enabled );
   bottom_->setEnabled( is_enabled );
   hcolor_->setEnabled( is_enabled );
   hwidth_->setEnabled( is_enabled );
   htrans_->setEnabled( is_enabled );
   horder_->setEnabled( is_enabled );

   if( top_->checkState() == Qt::Checked ) {
      top_color_    ->setEnabled( is_enabled );
      top_color_btn_->setEnabled( is_enabled );
      top_width_    ->setEnabled( is_enabled );
      top_trans_    ->setEnabled( is_enabled );   
      top_order_    ->setEnabled( is_enabled );   
   }
   if( bottom_->checkState() == Qt::Checked ) {
      bottom_color_    ->setEnabled( is_enabled );
      bottom_color_btn_->setEnabled( is_enabled );
      bottom_width_    ->setEnabled( is_enabled );
      bottom_trans_    ->setEnabled( is_enabled );
      bottom_order_    ->setEnabled( is_enabled );
   }
}
// end of horizontal_changed

//*******************************************************************
// top_changed                                          PRIVATE slot
//*******************************************************************
void QBtConfigBorder::top_changed( const int state )
{
   const bool is_enabled = ( Qt::Checked == state );
   top_color_    ->setEnabled( is_enabled );
   top_color_btn_->setEnabled( is_enabled );
   top_width_    ->setEnabled( is_enabled );
   top_trans_    ->setEnabled( is_enabled );   
   top_order_    ->setEnabled( is_enabled );   
}
// end of top_changed

//*******************************************************************
// bottom_changed                                       PRIVATE slot
//*******************************************************************
void QBtConfigBorder::bottom_changed( const int state )
{
   const bool is_enabled = ( Qt::Checked == state );
   bottom_color_    ->setEnabled( is_enabled );
   bottom_color_btn_->setEnabled( is_enabled );
   bottom_width_    ->setEnabled( is_enabled );
   bottom_trans_    ->setEnabled( is_enabled );
   bottom_order_    ->setEnabled( is_enabled );
}
// end of bottom_changed

//*******************************************************************
// vertical_changed                                     PRIVATE slot
//*******************************************************************
void QBtConfigBorder::vertical_changed( const int state )
{
   const bool is_enabled = ( Qt::Checked == state );
   left_->setEnabled( is_enabled );
   right_->setEnabled( is_enabled );
   vcolor_->setEnabled( is_enabled );
   vwidth_->setEnabled( is_enabled );
   vtrans_->setEnabled( is_enabled );
   vorder_->setEnabled( is_enabled );

   if( left_->checkState() == Qt::Checked ) {
      left_color_    ->setEnabled( is_enabled );
      left_color_btn_->setEnabled( is_enabled );
      left_width_    ->setEnabled( is_enabled );
      left_trans_    ->setEnabled( is_enabled );
      left_order_    ->setEnabled( is_enabled );
   }
   if( right_->checkState() == Qt::Checked ) {
      right_color_    ->setEnabled( is_enabled );
      right_color_btn_->setEnabled( is_enabled );
      right_width_    ->setEnabled( is_enabled );
      right_trans_    ->setEnabled( is_enabled );
      right_order_    ->setEnabled( is_enabled );
   }
}
// end of vertical_changed

//*******************************************************************
// left_changed                                         PRIVATE slot
//*******************************************************************
void QBtConfigBorder::left_changed( const int state )
{
   const bool is_enabled = ( Qt::Checked == state );
   left_color_    ->setEnabled( is_enabled );
   left_color_btn_->setEnabled( is_enabled );
   left_width_    ->setEnabled( is_enabled );
   left_trans_    ->setEnabled( is_enabled );
   left_order_    ->setEnabled( is_enabled );
}
// end of left_changed

//*******************************************************************
// right_changed                                        PRIVATE slot
//*******************************************************************
void QBtConfigBorder::right_changed( const int state )
{
   const bool is_enabled = ( Qt::Checked == state );
   right_color_    ->setEnabled( is_enabled );
   right_color_btn_->setEnabled( is_enabled );
   right_width_    ->setEnabled( is_enabled );
   right_trans_    ->setEnabled( is_enabled );
   right_order_    ->setEnabled( is_enabled );
}
// end of right_changed

//*******************************************************************
// top_color_clicked                                    PRIVATE slot
//*******************************************************************
void QBtConfigBorder::top_color_clicked()
{
   const QColor new_color = QColorDialog::getColor( top_color_->background(), this );
   if( new_color.isValid() ) {
      top_color_->background( new_color );
   }
}
// end of top_color_clicked

//*******************************************************************
// bottom_color_clicked                                 PRIVATE slot
//*******************************************************************
void QBtConfigBorder::bottom_color_clicked()
{
   const QColor new_color = QColorDialog::getColor( bottom_color_->background(), this );
   if( new_color.isValid() ) {
      bottom_color_->background( new_color );
   }
}
// end of bottom_color_clicked

//*******************************************************************
// left_color_clicked                                   PRIVATE slot
//*******************************************************************
void QBtConfigBorder::left_color_clicked()
{
   const QColor new_color = QColorDialog::getColor( left_color_->background(), this );
   if( new_color.isValid() ) {
      left_color_->background( new_color );
   }
}
// end of left_color_clicked

//*******************************************************************
// right_color_clicked                                  PRIVATE slot
//*******************************************************************
void QBtConfigBorder::right_color_clicked()
{
   const QColor new_color = QColorDialog::getColor( right_color_->background(), this );
   if( new_color.isValid() ) {
      right_color_->background( new_color );
   }
}
// end of right_color_clicked
 