#ifndef INCLUDED_QBtConfigBorder_h
#define INCLUDED_QBtConfigBorder_h
/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtConfigBorder.h
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 14.04.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtConfigBaseDialog.h"

/*------- forward declarations:
-------------------------------------------------------------------*/
class QCheckBox;
class QSpinBox;
class QLabel;
class QBtColorDemo;

/*------- class declaration:
-------------------------------------------------------------------*/
class QBtConfigBorder : public QBtConfigBaseDialog
{
   Q_OBJECT

//------- CONSTRUCTION/DESTRUCTION -------
public:
   QBtConfigBorder( QWidget*, const QString& );
   ~QBtConfigBorder();
private:
   QBtConfigBorder( const QBtConfigBorder& );
   QBtConfigBorder& operator=( const QBtConfigBorder& );

//------- CONSTANTS -------
private:
   static const char* const Caption;
   static const char* const Horizontal;
   static const char* const Vertical;
   static const char* const Top;
   static const char* const Bottom;
   static const char* const Left;
   static const char* const Right;
   static const char* const Width;
   static const char* const Transparency;
   static const char* const Order;
   static const char* const Color;

//------- MEMBERS -------
private:
   QCheckBox   * const horizontal_;
   QCheckBox   * const top_;
   QCheckBox   * const bottom_;
   QCheckBox   * const vertical_;
   QCheckBox   * const left_;
   QCheckBox   * const right_;
   //..................................
   QBtColorDemo* const top_color_;
   QBtColorDemo* const bottom_color_;
   QBtColorDemo* const left_color_;
   QBtColorDemo* const right_color_;
   //..................................
   QPushButton * const top_color_btn_;
   QPushButton * const bottom_color_btn_;
   QPushButton * const left_color_btn_;
   QPushButton * const right_color_btn_;
   //..................................
   QSpinBox    * const top_width_;
   QSpinBox    * const bottom_width_;
   QSpinBox    * const left_width_;
   QSpinBox    * const right_width_;
   //..................................
   QSpinBox    * const top_trans_;
   QSpinBox    * const bottom_trans_;
   QSpinBox    * const left_trans_;
   QSpinBox    * const right_trans_;
   //..................................
   QSpinBox    * const top_order_;
   QSpinBox    * const bottom_order_;
   QSpinBox    * const left_order_;
   QSpinBox    * const right_order_;
   //..................................
   QLabel      * const hcolor_;
   QLabel      * const vcolor_;
   QLabel      * const hwidth_;
   QLabel      * const vwidth_;
   QLabel      * const htrans_;
   QLabel      * const vtrans_;
   QLabel      * const horder_;
   QLabel      * const vorder_;

//------- METHODS -------
private:
   void showEvent( QShowEvent* );
   void customEvent( QEvent* );
   QString get_command() const;
   void update_values( const QString& );
private slots:
   void try_clicked();
   void accept_clicked();
   void horizontal_changed( int );
   void top_changed( int );
   void bottom_changed( int );
   void vertical_changed( int );
   void left_changed( int );
   void right_changed( int );
   void top_color_clicked();
   void bottom_color_clicked();
   void left_color_clicked();
   void right_color_clicked();
};

#endif // INCLUDED_QBtConfigBorder_h
