/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtConfigGrayScale.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 11.04.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtConfigGrayScale.h"
#include "QBtEventsController.h"
#include "QBtShared.h"
#include <QPushButton>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtConfigGrayScale::Caption = QT_TR_NOOP( "Gray Scale" );


//*******************************************************************
// QBtConfigGrayScale                                    CONSTRUCTOR
//*******************************************************************
QBtConfigGrayScale::QBtConfigGrayScale( QWidget* const parent, const QString& fpath )
: QBtConfigBaseDialog( parent, fpath )
{
   setWindowTitle( tr( Caption ) );

   connect( try_   , SIGNAL( clicked() ), this, SLOT( try_clicked   () ) );
   connect( accept_, SIGNAL( clicked() ), this, SLOT( accept_clicked() ) );

   QBtEventsController* const evc = QBtEventsController::instance();
   evc->append( this, QBtEvent::CommandsAndParamsAnswer );
}
// end of QBtConfigGrayScale

//*******************************************************************
// ~QBtConfigGrayScale                                    DESTRUCTOR
//*******************************************************************
QBtConfigGrayScale::~QBtConfigGrayScale()
{
   QBtEventsController::instance()->remove( this );
}
// end of ~QBtConfigGrayScale

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtConfigGrayScale::customEvent( QEvent* const e )
{
   QBtEvent* const event = dynamic_cast<QBtEvent*>( e );
   const int type = static_cast<int>( event->type() );

   switch( type ) {
      case QBtEvent::CommandsAndParamsAnswer:
         try_all( event->data( 0 ).toStringList(), "grayscale", get_command() );
   }
}
// end of customEvent

//*******************************************************************
// get_command                                     PRIVATE inherited
//*******************************************************************
QString QBtConfigGrayScale::get_command() const
{
   return QString( "yes" );
}
// end of get_command

//*******************************************************************
// try_clicked                                          PRIVATE slot
//*******************************************************************
void QBtConfigGrayScale::try_clicked()
{
   bool error = false;
   
   QBtShared::disable_updates( this );
   try {
      QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
      QBtMagic tmp = magic_;
      tmp.grayscale( get_command() );
      dst_qimage_ = tmp.qimage();
      resize_image( false );
   }
   catch( Magick::Warning& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::OutputMessage, QString( e.what() ) );
      error = true;
   }
   catch( Magick::Error& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( e.what() ) );
      error = true;
   }
   catch(...) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( "???" ) );
      error = true;
   }
   QBtShared::enable_updates( this );

   if( error ) reject();

}
// end of try_clicked

//*******************************************************************
// accept_clicked                                       PRIVATE slot
//*******************************************************************
void QBtConfigGrayScale::accept_clicked()
{
   QBtEventsController::instance()->send_event( QBtEvent::SaveCommand, "grayscale", get_command() );
   accept(); 
}
// end of accept_clicked
 