/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtConfigResize.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 31.03.2009
 *******************************************************************/

 /*------- include files:
 ------------------------------------------------------------------*/
 #include "Parsery.h"
 #include "QBtShared.h"
 #include "QBtEventsController.h"
 #include "QBtConfigResize.h"
 #include <QLabel>
 #include <QHBoxLayout>
 #include <QGridLayout>
 #include <QRadioButton>
 #include <QCheckBox>
 #include <QButtonGroup>
 #include <QSpinBox>
 #include <QPushButton>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtConfigResize::Caption     = QT_TR_NOOP( "Resize" );
const char* const QBtConfigResize::Value       = QT_TR_NOOP( "&Value" );
const char* const QBtConfigResize::AspectRatio = QT_TR_NOOP( "&aspect ratio" );
const char* const QBtConfigResize::Percent     = QT_TR_NOOP( "&Percent" );
const char* const QBtConfigResize::Options     = QT_TR_NOOP( "Options:" );
const char* const QBtConfigResize::Transform   = QT_TR_NOOP( "&transform algorithm" );
const char* const QBtConfigResize::Scale       = QT_TR_NOOP( "simple &ratio algorithm" );
const char* const QBtConfigResize::Px          = QT_TR_NOOP( "px" );

//*******************************************************************
// QBtConfigResize                                       CONSTRUCTOR
//*******************************************************************
QBtConfigResize::QBtConfigResize( QWidget* parent, const QString& fpath )
: QBtConfigBaseDialog( parent, fpath )
, value_radio_       ( new QRadioButton( tr( Value ) ) )
, percent_radio_     ( new QRadioButton( tr( Percent ) ) )
, options_           ( new QLabel( tr( Options ) ) )
, aspect_ratio_cbx_  ( new QCheckBox( tr( AspectRatio ) ) )
, x_spin_box_        ( new QSpinBox )
, y_spin_box_        ( new QSpinBox )
, percent_spin_box_  ( new QSpinBox )
, transform_radio_   ( new QRadioButton( tr( Transform ) ) )
, scale_radio_       ( new QRadioButton( tr( Scale ) ) )
{
   setWindowTitle( tr( Caption ) );

   QButtonGroup* const bg1 = new QButtonGroup( this );
   bg1->addButton( value_radio_ );
   bg1->addButton( percent_radio_ );
   QButtonGroup* const bg2 = new QButtonGroup( this );
   bg2->addButton( transform_radio_ );
   bg2->addButton( scale_radio_ );

   value_radio_->setChecked( true );
   aspect_ratio_cbx_->setCheckState( Qt::Checked );
   x_spin_box_->setSuffix( tr( Px ) );
   y_spin_box_->setSuffix( tr( Px ) );
   x_spin_box_->setSingleStep( 1 );
   y_spin_box_->setSingleStep( 1 );
   //............................................
   transform_radio_->setChecked( true );
   //............................................
   percent_spin_box_->setEnabled( false );
   percent_spin_box_->setRange( 1, 100 );
   percent_spin_box_->setSuffix( "%" );
   percent_spin_box_->setValue( 50 );
   
   QGridLayout* const xy_grid = new QGridLayout;
   xy_grid->addWidget( aspect_ratio_cbx_, 0, 1 );
   xy_grid->addWidget( x_spin_box_      , 1, 1 );
   xy_grid->addWidget( y_spin_box_      , 2, 1 );
   xy_grid->setColumnMinimumWidth( 0, 20 );
   
   QGridLayout* const percent_grid = new QGridLayout;
   percent_grid->addWidget( percent_spin_box_, 0, 1 );
   percent_grid->setColumnMinimumWidth( 0, 20 );

   QVBoxLayout* const options_layout = new QVBoxLayout;
   options_layout->addWidget( transform_radio_ );
   options_layout->addWidget( scale_radio_ );
   QGridLayout* const options_grid = new QGridLayout;
   options_grid->addLayout( options_layout, 0, 1 );
   options_grid->setColumnMinimumWidth( 0, 20 );

   QGridLayout* const grid = new QGridLayout;
   grid->addWidget( value_radio_  , 0, 0, 1, 1, Qt::AlignTop | Qt::AlignLeft );
   grid->addWidget( percent_radio_, 0, 2, 1, 1, Qt::AlignTop | Qt::AlignLeft );
   grid->addWidget( options_      , 0, 4, 1, 1, Qt::AlignTop | Qt::AlignLeft );
   grid->addLayout( xy_grid       , 1, 0, 1, 1, Qt::AlignTop | Qt::AlignLeft );
   grid->addLayout( percent_grid  , 1, 2, 1, 1, Qt::AlignTop | Qt::AlignLeft );
   grid->addLayout( options_grid  , 1, 4, 1, 1, Qt::AlignTop | Qt::AlignLeft );
   grid->setColumnMinimumWidth( 1, 20 );
   grid->setColumnMinimumWidth( 3, 20 );

   QHBoxLayout* const my_layout = new QHBoxLayout;
   my_layout->addLayout( grid );
   my_layout->addStretch();
   user_layout_->addLayout( my_layout );

   connect( try_, SIGNAL(clicked()), this, SLOT( try_clicked()));
   connect( accept_, SIGNAL(clicked()), this, SLOT( accept_clicked()));
   connect( value_radio_, SIGNAL(toggled(bool)), this, SLOT(value_radio_toggled(bool)));
   connect( percent_radio_, SIGNAL(toggled(bool)), this, SLOT(percent_radio_toggled(bool)));
   connect( aspect_ratio_cbx_, SIGNAL(stateChanged(int)), this, SLOT(aspect_ratio_changed()));
   connect( x_spin_box_, SIGNAL(valueChanged(int)), this, SLOT(x_value_changed()));
   connect( y_spin_box_, SIGNAL(valueChanged(int)), this, SLOT(y_value_changed()));

   QBtEventsController* const evc = QBtEventsController::instance();
   evc->append( this, QBtEvent::ParamsAnswer );
   evc->append( this, QBtEvent::CommandsAndParamsAnswer );
}
// end of QBtConfigResize

//*******************************************************************
// ~QBtConfigResize                                       DESTRUCTOR
//*******************************************************************
QBtConfigResize::~QBtConfigResize()
{
   QBtEventsController::instance()->remove( this );
}
// end of ~QBtConfigResize

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void QBtConfigResize::showEvent( QShowEvent* const e )
{
   QBtConfigBaseDialog::showEvent( e );
   
   const int w = magic_.width();
   const int h = magic_.height();
   x_spin_box_->setRange( 10, w );
   y_spin_box_->setRange( 10, h ); 
   x_spin_box_->setValue( w );
   y_spin_box_->setValue( h );

   QBtEventsController::instance()->send_event( QBtEvent::ParamsRequest, "resize" );
}
// end of showEvent

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtConfigResize::customEvent( QEvent* const e )
{
   QBtEvent* const event = dynamic_cast<QBtEvent*>( e );
   const int type = static_cast<int>( event->type() );

   switch( type ) {
      case QBtEvent::ParamsAnswer:
         update_values( event->data( 0 ).toString() );
         break;
      case QBtEvent::CommandsAndParamsAnswer:
         try_all( event->data( 0 ).toStringList(), "resize", get_command() );
   }
}
// end of customEvent

//*******************************************************************
// update_values                                             PRIVATE
//*******************************************************************
void QBtConfigResize::update_values( const QString& params )
{
   CBtResizeParser parser( params );
   if( !parser.valid() ) return;

   if( parser.newsize() ) {
      x_spin_box_->setValue( parser.width () );
      y_spin_box_->setValue( parser.height() );
   }
   else {
      percent_spin_box_->setValue( parser.percent() );
   }
   if( parser.scale() ) {
      scale_radio_->setChecked( true );
   }
}
// end of update_values

//*******************************************************************
// value_radio_toggled                                  PRIVATE slot
//*******************************************************************
void QBtConfigResize::value_radio_toggled( bool state )
{
   aspect_ratio_cbx_->setEnabled( state );
   x_spin_box_->setEnabled( state );
   y_spin_box_->setEnabled( state );
}
// end of percent_radio_toogled

//*******************************************************************
// percent_radio_toggled                                PRIVATE slot
//*******************************************************************
void QBtConfigResize::percent_radio_toggled( bool state )
{
   percent_spin_box_->setEnabled( state );
}
// end of percent_radio_toogled

//*******************************************************************
// aspect_ratio_changed                                 PRIVATE slot
//*******************************************************************
void QBtConfigResize::aspect_ratio_changed()
{ 
   const bool flag = ( Qt::Checked == aspect_ratio_cbx_->checkState() );
   if( flag ) y_recalculate();
}
// end of aspect_ratio_changed

//*******************************************************************
// x_value_changed                                      PRIVATE slot
//*******************************************************************
void QBtConfigResize::x_value_changed()
{
   if( Qt::Checked == aspect_ratio_cbx_->checkState() ) {
      y_recalculate();
   }
}
// end of x_value_changed

//*******************************************************************
// y_value_changed                                      PRIVATE slot
//*******************************************************************
void QBtConfigResize::y_value_changed()
{
   if( Qt::Checked == aspect_ratio_cbx_->checkState() ) {
      x_recalculate();
   }
}
// end of y_value_changed

//*******************************************************************
// x_recalculate                                             PRIVATE
//*******************************************************************
void QBtConfigResize::x_recalculate()
{
   const int w        = magic_.width();
   const int h        = magic_.height();
   const int y        = y_spin_box_->value();
   const double ratio = static_cast<double>( w ) / static_cast<double>( h );

   disconnect( x_spin_box_, SIGNAL(valueChanged(int)), this, SLOT(x_value_changed()));
   x_spin_box_->setValue( static_cast<int>( y * ratio ) );
   connect( x_spin_box_, SIGNAL(valueChanged(int)), this, SLOT(x_value_changed()));
}
// end of x_recalculate

//*******************************************************************
// y_recalculate                                             PRIVATE
//*******************************************************************
void QBtConfigResize::y_recalculate()
{
   const int w        = magic_.width();
   const int h        = magic_.height();
   const int x        = x_spin_box_->value();
   const double ratio = static_cast<double>( h ) / static_cast<double>( w );

   disconnect( y_spin_box_, SIGNAL(valueChanged(int)), this, SLOT(y_value_changed()));
   y_spin_box_->setValue( static_cast<int>( x * ratio ) );
   connect( y_spin_box_, SIGNAL(valueChanged(int)), this, SLOT(y_value_changed()));
}
// end of y_recalculate

//*******************************************************************
// get_command                                     PRIVATE inherited
//*******************************************************************
QString QBtConfigResize::get_command() const
{
   CBtResizeParser parser;

   if( value_radio_->isChecked() ) {
      parser.width ( x_spin_box_->value() );
      parser.height( y_spin_box_->value() );
   }
   else {
      parser.percent( percent_spin_box_->value() );
   }
   if( scale_radio_->isChecked() ) {
      parser.use_scale();
   }
   return parser.command();
}
// end of get_command

//*******************************************************************
// try_clicked                                          PRIVATE slot
//*******************************************************************
void QBtConfigResize::try_clicked()
{
   bool error = false;
   
   QBtShared::disable_updates( this );
   try {
      QBtEventsController::instance()->send_event( QBtEvent::ClearMessages );
      QBtMagic tmp = magic_;
      tmp.resize( get_command() );
      dst_qimage_ = tmp.qimage();
      resize_image( false );
   }
   catch( Magick::Warning& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::OutputMessage, QString( e.what() ) );
      error = true;
   }
   catch( Magick::Error& e ) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( e.what() ) );
      error = true;
   }
   catch(...) {
      QBtEventsController::instance()->send_event( QBtEvent::ErrorMessage, QString( "???" ) );
      error = true;
   }
   QBtShared::enable_updates( this );

   if( error ) reject();
}
// end of try_clicked

//*******************************************************************
// accept_clicked                                       PRIVATE slot
//*******************************************************************
void QBtConfigResize::accept_clicked()
{
   QBtEventsController::instance()->send_event( QBtEvent::SaveCommand, "resize", get_command() );
   accept(); 
}
// end of accept_clicked
