#ifndef INCLUDED_QBtConfigResize_h
#define INCLUDED_QBtConfigResize_h
/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtConfigResize.h
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 31.03.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtConfigBaseDialog.h"

/*------- forward declarations:
-------------------------------------------------------------------*/
class QRadioButton;
class QLabel;
class QSpinBox;
class QCheckBox;

/*------- class declaration:
-------------------------------------------------------------------*/
class QBtConfigResize : public QBtConfigBaseDialog
{
   Q_OBJECT

//------- CONSTRUCTION / DESTRUCTION -------
public:
   QBtConfigResize( QWidget*, const QString& );
   ~QBtConfigResize();
private:
   QBtConfigResize( const QBtConfigResize& );
   QBtConfigResize& operator=( const QBtConfigResize& );

//------- CONSTANTS -------
private:
   static const char* const Caption;
   static const char* const Percent;
   static const char* const Value;
   static const char* const AspectRatio;
   static const char* const Options;
   static const char* const Transform;
   static const char* const Scale;
   static const char* const Px;

//------- MEMBERS -------
private:
   QRadioButton* const value_radio_;
   QRadioButton* const percent_radio_;
   QLabel      * const options_;
   QCheckBox   * const aspect_ratio_cbx_;
   QSpinBox    * const x_spin_box_;
   QSpinBox    * const y_spin_box_;
   QSpinBox    * const percent_spin_box_;
   QRadioButton* const transform_radio_;
   QRadioButton* const scale_radio_;
   
//------- METHODS -------
private:
   void    showEvent( QShowEvent* );
   void    customEvent( QEvent* );
   void    update_values( const QString& );
   void    x_recalculate();
   void    y_recalculate();
   QString get_command() const;
private slots:
   void    try_clicked();
   void    accept_clicked();
   void    value_radio_toggled( bool );
   void    percent_radio_toggled( bool );
   void    aspect_ratio_changed();
   void    x_value_changed();
   void    y_value_changed();
};

#endif // INCLUDED_QBtConfigResize_h
 