#ifndef INCLUDED_QBtConfigWatermark_h
#define INCLUDED_QBtConfigWatermark_h
/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtConfigWatermark.h
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 28.03.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtConfigBaseDialog.h"

/*------- forward declarations:
-------------------------------------------------------------------*/
class QLabel;
class QLineEdit;
class QPushButton;
class QCheckBox;
class QSpinBox;
class QRadioButton;
class QGroupButton;

/*------- class declaration:
-------------------------------------------------------------------*/
class QBtConfigWatermark : public QBtConfigBaseDialog
{
   Q_OBJECT

//------- CONSTRUCTION / DESTRUCTION -------
public:
   QBtConfigWatermark( QWidget*, const QString& );
   ~QBtConfigWatermark();
private:
   QBtConfigWatermark( const QBtConfigWatermark& );
   QBtConfigWatermark& operator=( const QBtConfigWatermark& );

//------- CONSTANTS -------
private:
   static const char* const Caption;
   static const char* const PathStr;
   static const char* const MarginStr;
   static const char* const ScaleStr;
   static const char* const XWidthStr;
   static const char* const YWidthStr;
   static const char* const GravityStr;
   static const char* const Transparent;
   static const char* const NorthWest;
   static const char* const North;
   static const char* const NorthEast;
   static const char* const West;
   static const char* const Center;
   static const char* const East;
   static const char* const SouthWest;
   static const char* const South;
   static const char* const SouthEast;

//------- MEMBERS -------
private:
   QLineEdit   * const picture_path_;
   QPushButton * const picture_path_button_;
   QLabel      * const gravity_label_;
   QLabel      * const trans_label_;
   QSpinBox    * const trans_spin_box_;
   QRadioButton* const nwg_rb_;
   QRadioButton* const ng_rb_;
   QRadioButton* const neg_rb_;
   QRadioButton* const wg_rb_;
   QRadioButton* const cg_rb_;
   QRadioButton* const eg_rb_;
   QRadioButton* const swg_rb_;
   QRadioButton* const sg_rb_;
   QRadioButton* const seg_rb_;
   QCheckBox   * const margin_check_box_;
   QLabel      * const x_margin_label_;
   QLabel      * const y_margin_label_;
   QSpinBox    * const x_margin_spin_box_;
   QSpinBox    * const y_margin_spin_box_;
   QCheckBox   * const scale_check_box_;
   QSpinBox    * const scale_spin_box_;

//------- METHODS -------
private:
   void showEvent          ( QShowEvent* );
   void customEvent        ( QEvent* );
   void update_values      ( const QString& );
   void set_margin_enabled ( bool );
   void set_gravity_enabled( bool );
   void set_scale_enabled  ( bool );
   QString get_command     () const;
   
private slots:
   void try_clicked();
   void accept_clicked();
   void margin_check_box_changed( int );
   void scale_check_box_changed( int );
   void picture_path_button_clicked();
   void picture_path_changed( const QString& );
};
#endif // INCLUDED_QBtConfigWatermark_h
