/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtInfo.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 05.02.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtInfo.h"
#include "QBtShared.h"
#include "QBtProcess.h"
//.....................................
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPushButton>
#include <QTextBrowser>
#include <QApplication>
#include <QDesktopWidget>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtInfo::Caption = QT_TR_NOOP( "Pictures metadata" );
const char* const QBtInfo::Close   = QT_TR_NOOP( "&Close" );

//*******************************************************************
// QBtInfo                                               CONSTRUCTOR
//*******************************************************************
QBtInfo::QBtInfo( QWidget* const in_parent, const QString& in_cmd )
: QDialog( in_parent )
, cmd_     ( in_cmd )
, browser_ ( new QTextBrowser )
, close_   ( new QPushButton( tr( Close ) ) )
, process_ ( new QBtProcess( this ) )
{
   setWindowTitle( tr( Caption ) );

   QHBoxLayout* const btn_layout = new QHBoxLayout;
   btn_layout->addStretch();
   btn_layout->addWidget( close_ );

   QVBoxLayout* const main_layout = new QVBoxLayout;
   main_layout->addWidget( browser_ );
   main_layout->addLayout( btn_layout );
   setLayout( main_layout );

   connect( process_, SIGNAL( finished() ), this, SLOT( ready_slot() ) );
   connect( close_  , SIGNAL( clicked () ), this, SLOT( accept() ) );
}
// end of QBtInfo

//*******************************************************************
// ~QBtInfo                                               DESTRUCTOR
//*******************************************************************
QBtInfo::~QBtInfo()
{
   if( process_ ) {
      delete process_;
      process_ = 0;
   }
}
// end of ~QBtInfo

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void QBtInfo::showEvent( QShowEvent* const in_event )
{
   QDialog::showEvent( in_event );

   const int   screen_width   = qApp->desktop()->availableGeometry().width();
   const int   screen_height  = qApp->desktop()->availableGeometry().height();
   const qreal width_percent  = 60.0 / 100.0;
   const qreal height_percent = 60.0 / 100.0;    
   const int   dx = static_cast<int>( screen_width  * width_percent );
   const int   dy = static_cast<int>( screen_height * height_percent );
   const int   x  = ( screen_width  - dx + 1 ) >> 1;
   const int   y  = ( screen_height - dy + 1 ) >> 1;
   move( x, y );
   resize( dx, dy );
}
// end of showEvent

//*******************************************************************
// focusInEvent                                    PRIVATE inherited
//*******************************************************************
void QBtInfo::focusInEvent( QFocusEvent* const in_event )
{
   QDialog::focusInEvent( in_event );
   QBtShared::disable_updates( this );
   static const QString cmd = "identify -verbose \"%1\"";
   process_->run( cmd.arg( cmd_ ) );   
}
// end of focusInEvent

//*******************************************************************
// ready                                                PRIVATE slot
//*******************************************************************
void QBtInfo::ready_slot()
{
   browser_->insertPlainText( process_->get_out_text() );
   QBtShared::enable_updates( this );
}
// end of ready
