#ifndef INCLUDED_QBtMagic_h
#define INCLUDED_QBtMagic_h
/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtMagic.h
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 10.02.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <Magick++.h>
#include <QObject>
#include <QImage>
#include <string>

class QColor;

class QBtMagic : public QObject
{
   Q_OBJECT

//------- TYPES -------
private:
   class CArea
   {
   public:
      CArea()
      : xlt_  ( -1 )
      , ylt_  ( -1 )
      , xrb_  ( -1 )
      , yrb_  ( -1 )
      , color_( 0, 0, 0, 0 )
      {}
      CArea( const CArea& rhs )
      : xlt_   ( rhs.xlt_   )
      , ylt_   ( rhs.ylt_   )
      , xrb_   ( rhs.xrb_   )
      , yrb_   ( rhs.yrb_   )
      , color_ ( rhs.color_ )
      {}
      CArea& operator=( const CArea& rhs )
      {
         if( this != &rhs ) {
            xlt_   = rhs.xlt_;
            ylt_   = rhs.ylt_;
            xrb_   = rhs.xrb_;
            yrb_   = rhs.yrb_;
            color_ = rhs.color_;
         }
         return *this;
      }
   private:
      int xlt_;
      int ylt_;
      int xrb_;
      int yrb_;
      Magick::Color color_;
   public:
      bool valid() const {
         if( xlt_ == -1 ) return false;
         if( ylt_ == -1 ) return false;
         if( xrb_ == -1 ) return false;
         if( yrb_ == -1 ) return false;
         return true;
      }
      int xlt() const { return xlt_; }
      int ylt() const { return ylt_; }
      int xrb() const { return xrb_; }
      int yrb() const { return yrb_; }
      const Magick::Color& color() const { return color_; }
      Magick::Color& color() { return color_; }

      void set_xlt( const int value ) { xlt_ = value; }
      void set_ylt( const int value ) { ylt_ = value; }
      void set_xrb( const int value ) { xrb_ = value; }
      void set_yrb( const int value ) { yrb_ = value; }
      void set_color( const Magick::Color& value ) { color_ = value; }
   };
   
//------- CONSTRUCTION / DESTRUCTION -------
public:
   QBtMagic( QWidget* );
   QBtMagic( const QBtMagic& );
   ~QBtMagic();
   QBtMagic& operator=( const QBtMagic& );

//------- MEMBERS -------
private:
   QWidget*      parent_;
   Magick::Image image_;
   QString       fdir_;
   QString       fname_;
   QString       fext_;
   bool          ok_;
   QImage        qimage_;
   bool          visual_;

//------- METHODS -------
public:
   bool    read          ( const QString& );
   bool    write         ();
   bool    write2dir     ( const QString& );
   bool    write2file    ( const QString& );
   bool    process       ( const QString& );
   bool    process       ( const QString&, const QString& );
   void    clear         ();
   void    set_format    ( const QString& );
   QImage& qimage        ();
   bool    valid         () const { return ok_; }
   void    grayscale     ( const QString& );
   void    negate        ( const QString& );
   void    normalize     ( const QString& );
   void    enhance       ( const QString& );
   void    equalize      ( const QString& );
   void    border        ( const QString& );
   void    watermark     ( const QString& );
   void    rotate        ( int );
   void    rotate        ( const QString& );
   void    annotate      ( const QString& );
   void    resize        ( const QString& );
public:
   int                 width                 () const { return image_.columns(); }
   int                 height                () const { return image_.rows();    }
   void                visual_on()  { visual_ = true;  }
   void                visual_off() { visual_ = false; }
private:
   void remember_fpath( const QString& );
   Magick::GravityType get_gravity_type      ( const QString& ) const;
   void ustaw_przezroczystosc ( Magick::Image&, double ) const;
};

#endif // INCLUDED_QBtMagic_h
