/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtShared.h
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 2.01.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtShared.h"
#include "QBtMainWindow.h"
#include "QBtWorkspace.h"
#include "QBtEventsController.h"
#include <QMenu>
#include <QAction>
#include <QMenuBar>
#include <QToolBar>
#include <QStatusBar>
#include <QApplication>
#include <QMessageBox>

/*------- constants:
-------------------------------------------------------------------*/
const char* const QBtMainWindow::MenuSystem    = QT_TR_NOOP( "&System" );
const char* const QBtMainWindow::MenuOper      = QT_TR_NOOP( "&Operations" );
const char* const QBtMainWindow::MenuHelp      = QT_TR_NOOP( "&Help" );
//---------------------------------------------------------
const char* const QBtMainWindow::Exit          = QT_TR_NOOP( "&Exit" );
const char* const QBtMainWindow::ExitShortcut  = QT_TR_NOOP( "Ctrl+Q" );
//---------------------------------------------------------
const char* const QBtMainWindow::Format        = QT_TR_NOOP( "&Format" );
const char* const QBtMainWindow::NoFormat      = QT_TR_NOOP( "&oryginal" );
const char* const QBtMainWindow::Gif           = QT_TR_NOOP( "&gif" );
const char* const QBtMainWindow::Jpeg          = QT_TR_NOOP( "&jpeg" );
const char* const QBtMainWindow::Png           = QT_TR_NOOP( "&png" );
const char* const QBtMainWindow::Tiff          = QT_TR_NOOP( "&tiff" );
const char* const QBtMainWindow::Bmp           = QT_TR_NOOP( "&bmp" );
//---------------------------------------------------------
const char* const QBtMainWindow::Color         = QT_TR_NOOP( "&Color" );
const char* const QBtMainWindow::Gray          = QT_TR_NOOP( "&Gray scale" );
const char* const QBtMainWindow::Negate        = QT_TR_NOOP( "&Negate" );
const char* const QBtMainWindow::Gamma         = QT_TR_NOOP( "Ga&mma" );
const char* const QBtMainWindow::Quantize      = QT_TR_NOOP( "&Quantize" );
const char* const QBtMainWindow::Normalize     = QT_TR_NOOP( "&Normalize" );
const char* const QBtMainWindow::Enhance       = QT_TR_NOOP( "M&inimize noise" );
const char* const QBtMainWindow::Equalize      = QT_TR_NOOP( "&Equalize" );

const char* const QBtMainWindow::Watermark    = QT_TR_NOOP( "&Watermark" );
const char* const QBtMainWindow::Rotate       = QT_TR_NOOP( "&Rotate" );
const char* const QBtMainWindow::Resize       = QT_TR_NOOP( "Re&size" );
const char* const QBtMainWindow::Border       = QT_TR_NOOP( "&Border" );
const char* const QBtMainWindow::Annotate     = QT_TR_NOOP( "&Annotate" );
//---------------------------------------------------------
const char* const QBtMainWindow::AboutProg    = QT_TR_NOOP( "&About program" );
const char* const QBtMainWindow::AboutQt      = QT_TR_NOOP( "About &Qt" );
//---------------------------------------------------------
const char* const QBtMainWindow::Ready        = QT_TR_NOOP( "Ready" );


//*******************************************************************
// QBtMainWindow                                         CONSTRUCTOR
//*******************************************************************
QBtMainWindow::QBtMainWindow() : QMainWindow()
, menu_system_( new QMenu )
, menu_oper_  ( new QMenu )
, menu_help_  ( new QMenu )
//-----------------------------------------------
, exit_ ( new QAction( QIcon( ":/img/exit.png" ), QString(), this ) )
//-----------------------------------------------
, format_      ( new QMenu )
, noformat_    ( new QAction( this ) )
, gif_         ( new QAction( this ) )
, jpeg_        ( new QAction( this ) )
, png_         ( new QAction( this ) )
, tiff_        ( new QAction( this ) )
, bmp_         ( new QAction( this ) )
//...............................................
, color_       ( new QMenu )
, gray_        ( new QAction( this ) )
, negate_      ( new QAction( this ) )
, gamma_       ( new QAction( this ) )
, quantize_    ( new QAction( this ) )
, normalize_   ( new QAction( this ) )
, enhance_     ( new QAction( this ) )
, equalize_    ( new QAction( this ) )
//...............................................
, watermark_   ( new QAction( QIcon( ":/img/watermark.png" ), QString(), this ) )
, rotate_      ( new QAction( QIcon( ":/img/rotate.png" ), QString(), this ) )
, resize_      ( new QAction( QIcon( ":/img/resize.png" ), QString(), this ) )
, border_      ( new QAction( QIcon( ":/img/border.png" ), QString(), this ) )
, annotate_    ( new QAction( QIcon( ":/img/fonts.png" ), QString(), this ) )
//-----------------------------------------------
, about_prog_ ( new QAction( QIcon( ":/img/magicom.png" ), QString(), this ) )
, about_qt_   ( new QAction( QIcon( ":/img/qt.png" ), QString(), this ) )
//-----------------------------------------------
, tbar_system_( new QToolBar )
{
   format_->setIcon( QIcon( ":/img/format.png" ) );
   color_->setIcon( QIcon( ":/img/color.png" ) );
   
   noformat_->setCheckable( true );
   noformat_->setChecked  ( true );
   gif_     ->setCheckable( true );
   jpeg_    ->setCheckable( true );
   png_     ->setCheckable( true );
   tiff_    ->setCheckable( true );
   bmp_     ->setCheckable( true );
   
   QActionGroup* const format_group = new QActionGroup( this );
   format_group->setExclusive( true );
   format_group->addAction( noformat_ );
   format_group->addAction( gif_ );
   format_group->addAction( jpeg_ );
   format_group->addAction( png_ );
   format_group->addAction( tiff_ );
   format_group->addAction( bmp_ );
   connect( format_group, SIGNAL( triggered(QAction*) ), this, SLOT( change_format(QAction*) ) );

   update_actions();
   create_menu();
   create_tool_bar();
   //create_status_bar();
   translate();

   setCentralWidget( new QBtWorkspace );

   QBtEventsController* const evc = QBtEventsController::instance();
   evc->append( this, QBtEvent::F10 );
}
// end of QBtMainWindow

//*******************************************************************
// QBtMainWindow                                          DESTRUCTOR
//*******************************************************************
QBtMainWindow::~QBtMainWindow()
{
   QBtEventsController::instance()->remove( this );
}
// end of ~QBtMainWindow

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void QBtMainWindow::showEvent( QShowEvent* const in_event )
{
   showMaximized();
   QMainWindow::showEvent( in_event );
}
// end of showEvent

//*******************************************************************
// closeEvent                                      PRIVATE inherited
//*******************************************************************
void QBtMainWindow::closeEvent( QCloseEvent* const in_event )
{
   QMainWindow::closeEvent( in_event );
}
// end of showEvent

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtMainWindow::customEvent( QEvent* const in_event )
{
   QBtEvent* const event = dynamic_cast<QBtEvent*>( in_event );
   const int type = static_cast<int>( event->type() );

   switch( type ) {
      case QBtEvent::F10:
         qApp->closeAllWindows();
         break;
   }
}
// end of customEvent

//*******************************************************************
// translate                                                 PRIVATE
//*******************************************************************
void QBtMainWindow::translate()
{
   setWindowTitle( QBtShared::program_name() );
   menu_system_->setTitle( tr( MenuSystem ) );
   menu_oper_  ->setTitle( tr( MenuOper ) );
   menu_help_  ->setTitle( tr( MenuHelp ) );
   //--------------------------------------------
   exit_->setText( tr( Exit ) );
   exit_->setShortcut( tr( ExitShortcut ) );
   //--------------------------------------------
   format_->setTitle( tr( Format ) );
   noformat_->setText( tr( NoFormat ) );
   gif_->setText( tr( Gif ) );
   jpeg_->setText( tr( Jpeg ) );
   png_->setText( tr( Png ) );
   tiff_->setText( tr( Tiff ) );
   bmp_->setText( tr( Bmp ) );
   //--------------------------------------------
   color_->setTitle( tr( Color ) );
   gray_->setText( tr( Gray ) );
   negate_->setText( tr( Negate ) );
   gamma_->setText( tr( Gamma ) );
   quantize_->setText( tr( Quantize ) );
   normalize_->setText( tr( Normalize ) );
   enhance_->setText( tr( Enhance ) );
   equalize_->setText( tr( Equalize ) );
   //--------------------------------------------
   watermark_->setText( tr( Watermark ) );
   rotate_->setText( tr( Rotate ) );
   resize_->setText( tr( Resize ) );
   border_->setText( tr( Border ) );
   annotate_->setText( tr( Annotate ) );
   //--------------------------------------------
   about_prog_->setText( tr( AboutProg ) );
   about_qt_->setText( tr( AboutQt ) );
}
// end of translate

//*******************************************************************
// create_menu                                               PRIVATE
//*******************************************************************
void QBtMainWindow::create_menu()
{
   menuBar()->addMenu( menu_system_ );
   menuBar()->addMenu( menu_oper_ );
   menuBar()->addMenu( menu_help_ );
   
   menu_system_->addAction( exit_ );

   menu_oper_->addMenu( format_ );
   format_->addAction( noformat_ );
   format_->addAction( gif_ );
   format_->addAction( jpeg_ );
   format_->addAction( png_ );
   format_->addAction( tiff_ );
   format_->addAction( bmp_ );

   menu_oper_->addMenu( color_ );
   color_->addAction( gray_ );
   color_->addAction( negate_ );
//   color_->addAction( gamma_ );
//   color_->addAction( quantize_ );
   color_->addSeparator();
   color_->addAction( normalize_ );
   color_->addAction( enhance_ );
   color_->addAction( equalize_ );
   
   menu_oper_->addAction( resize_ );
   menu_oper_->addAction( border_ );
   menu_oper_->addAction( annotate_ );
   menu_oper_->addAction( watermark_ );
   menu_oper_->addAction( rotate_ );

   menu_help_->addAction( about_prog_ );
   menu_help_->addAction( about_qt_ );
}
// end of create_menu

//*******************************************************************
// create_tool_bar                                           PRIVATE
//*******************************************************************
void QBtMainWindow::create_tool_bar()
{
   addToolBar( tbar_system_ );
   tbar_system_->addAction( exit_ );
}
// end of create_tool_bar

//*******************************************************************
// create_status_bar                                         PRIVATE
//*******************************************************************
void QBtMainWindow::create_status_bar()
{
   statusBar()->showMessage( tr( Ready ) );
}
// end of create_status_bar

//*******************************************************************
// update_actions                                            PRIVATE
//*******************************************************************
void QBtMainWindow::update_actions()
{
   connect( rotate_    , SIGNAL( triggered() ), this, SLOT( rotate         () ) );
   connect( gray_      , SIGNAL( triggered() ), this, SLOT( grayscale      () ) );
   connect( negate_    , SIGNAL( triggered() ), this, SLOT( negate         () ) );
   connect( normalize_ , SIGNAL( triggered() ), this, SLOT( normalize      () ) );
   connect( enhance_   , SIGNAL( triggered() ), this, SLOT( enhance        () ) );
   connect( equalize_  , SIGNAL( triggered() ), this, SLOT( equalize       () ) );
   connect( watermark_ , SIGNAL( triggered() ), this, SLOT( watermark      () ) );
   connect( resize_    , SIGNAL( triggered() ), this, SLOT( resize         () ) );
   connect( border_    , SIGNAL( triggered() ), this, SLOT( border         () ) );
   connect( annotate_  , SIGNAL( triggered() ), this, SLOT( annotate       () ) );
   connect( about_prog_, SIGNAL( triggered() ), this, SLOT( about_prog     () ) );
   connect( about_qt_  , SIGNAL( triggered() ), this, SLOT( about_qt       () ) );
   connect( exit_      , SIGNAL( triggered() ), qApp, SLOT( closeAllWindows() ) );
}
// end of update_actions

//*******************************************************************
// about_prog                                           PRIVATE slot
//*******************************************************************
void QBtMainWindow::about_prog()
{
   static const char* const text = QT_TR_NOOP(
      "<qt>"
      "<font color=blue>Image Commander is a program for bulk picture processing.</font>"
      "<hr>"
      "Author: Piotr Pszczolkowski<br>"
      "Contact: piotr@beesoft.org<br>"
      "Home page: www.beesoft.org"
      "</qt>" );
   QMessageBox::about( this, QBtShared::program_name(), tr( text ) );
}
// end of about_prog

//*******************************************************************
// about_qt                                             PRIVATE slot
//*******************************************************************
void QBtMainWindow::about_qt()
{
   QMessageBox::aboutQt( this );
}
// end of about_qt

//*******************************************************************
// change_format                                        PRIVATE slot
//*******************************************************************
void QBtMainWindow::change_format( QAction* const action )
{
   QString ext = QString();
        if( gif_  == action ) ext = "gif";
   else if( jpeg_ == action ) ext = "jpeg";
   else if( png_  == action ) ext = "png";
   else if( tiff_ == action ) ext = "tiff";
   else if( bmp_  == action ) ext = "bmp";

   QBtEventsController* const evc = QBtEventsController::instance();
   evc->send_event( QBtEvent::SaveCommand, "format", ext );
}
// end of change_format

//*******************************************************************
// negate                                               PRIVATE slot
//*******************************************************************
void QBtMainWindow::negate()
{
   QBtEventsController::instance()->send_event( QBtEvent::NegateRequest );
}
// end of negate

//*******************************************************************
// grayscale                                            PRIVATE slot
//*******************************************************************
void QBtMainWindow::grayscale()
{
   QBtEventsController::instance()->send_event( QBtEvent::GrayscaleRequest );
}
// end of grayscale

//*******************************************************************
// normalize                                            PRIVATE slot
//*******************************************************************
void QBtMainWindow::normalize()
{
   QBtEventsController::instance()->send_event( QBtEvent::NormalizeRequest );
}
// end of normalize

//*******************************************************************
// enhance                                              PRIVATE slot
//*******************************************************************
void QBtMainWindow::enhance()
{
   QBtEventsController::instance()->send_event( QBtEvent::EnhanceRequest );
}
// end of enhance

//*******************************************************************
// equalize                                             PRIVATE slot
//*******************************************************************
void QBtMainWindow::equalize()
{
   QBtEventsController::instance()->send_event( QBtEvent::EqualizeRequest );
}
// end of equalize

//*******************************************************************
// watermark                                            PRIVATE slot
//*******************************************************************
void QBtMainWindow::watermark()
{
   QBtEventsController::instance()->send_event( QBtEvent::WatermarkRequest );
}
// end of watermark

//*******************************************************************
// rotate                                               PRIVATE slot
//*******************************************************************
void QBtMainWindow::rotate()
{
   QBtEventsController::instance()->send_event( QBtEvent::RotateRequest );
}
// end of rotate

//*******************************************************************
// resize                                               PRIVATE slot
//*******************************************************************
void QBtMainWindow::resize()
{
   QBtEventsController::instance()->send_event( QBtEvent::ResizeRequest );
}
// end of resize

//*******************************************************************
// border                                               PRIVATE slot
//*******************************************************************
void QBtMainWindow::border()
{
   QBtEventsController::instance()->send_event( QBtEvent::BorderRequest );
}
// end of annotate

//*******************************************************************
// annotate                                             PRIVATE slot
//*******************************************************************
void QBtMainWindow::annotate()
{
   QBtEventsController::instance()->send_event( QBtEvent::AnnotateRequest );
}
// end of annotate
