#ifndef INCLUDED_QBtMainWindow_h
#define INCLUDED_QBtMainWindow_h
/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtMainWindow.h
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 2.01.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <QMainWindow>

/*------- forward declarations:
-------------------------------------------------------------------*/
class QMenu;
class QAction;
class QToolBar;

/*------- class declaration:
-------------------------------------------------------------------*/
class QBtMainWindow : public QMainWindow
{
   Q_OBJECT

//******* CONSTRUCTION / DESTRUCTION *******
public:
   QBtMainWindow();
   ~QBtMainWindow();
private:
   QBtMainWindow( const QBtMainWindow& );
   QBtMainWindow& operator=( const QBtMainWindow& );

//******* CONSTANTS *******
private:
   static const char* const MenuSystem;
   static const char* const MenuOper;
   static const char* const MenuHelp;
   //--------------------------------------------
   static const char* const Format;
   static const char* const NoFormat;
   static const char* const Gif;
   static const char* const Jpeg;
   static const char* const Png;
   static const char* const Tiff;
   static const char* const Bmp;
   //--------------------------------------------
   static const char* const Color;
   static const char* const Gray;
   static const char* const Negate;
   static const char* const Gamma;
   static const char* const Quantize;
   static const char* const Normalize;
   static const char* const Enhance;
   static const char* const Equalize;
   //--------------------------------------------
   static const char* const Watermark;
   static const char* const Rotate;
   static const char* const Resize;
   static const char* const Border;
   static const char* const Annotate;
   //--------------------------------------------
   static const char* const Exit;
   static const char* const ExitShortcut;
   //--------------------------------------------
   static const char* const AboutProg;
   static const char* const AboutQt;
   //--------------------------------------------
   static const char* const Ready;

//******* MEMBERS *******
private:
   QMenu* const menu_system_;
   QMenu* const menu_oper_;
   QMenu* const menu_help_;
   //--------------------------------------------
   QAction* const exit_;
   //--------------------------------------------
   QMenu* const format_;
   QAction* const noformat_;
   QAction* const gif_;
   QAction* const jpeg_;
   QAction* const png_;
   QAction* const tiff_;
   QAction* const bmp_;
   //........................
   QMenu  * const color_;
   QAction* const gray_;
   QAction* const negate_;
   QAction* const gamma_;
   QAction* const quantize_;
   QAction* const normalize_;
   QAction* const enhance_;
   QAction* const equalize_;
   
   QAction* const watermark_;
   QAction* const rotate_;
   QAction* const resize_;
   QAction* const border_;
   QAction* const annotate_;
   //--------------------------------------------
   QAction* const about_prog_;
   QAction* const about_qt_;
   //--------------------------------------------
   QToolBar* const tbar_system_;

//******* METHODS *******
private:
   void showEvent         ( QShowEvent*  );
   void closeEvent        ( QCloseEvent* );
   void customEvent       ( QEvent* );
   void translate         ();
   void create_menu       ();
   void create_tool_bar   ();
   void create_status_bar ();
   void update_actions    ();
private slots:
   void about_prog    ();
   void about_qt      ();
   void change_format ( QAction* );
   void negate        ();
   void grayscale     ();
   void normalize     ();
   void enhance       ();
   void equalize      ();
   void watermark     ();
   void rotate        ();
   void resize        ();
   void border        ();
   void annotate      ();
};

 #endif // INCLUDED_QBtMainWindow_h
