/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtOutputWidget.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 8.01.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtOutputWidget.h"
#include "QBtEventsController.h"
#include <QVBoxLayout>
#include <QLabel>
#include <QSplitter>
#include <QTextBrowser>

//*******************************************************************
// QBtOutputWidget                                          CONSTRUCTOR
//*******************************************************************
QBtOutputWidget::QBtOutputWidget( QWidget* const in_parent )
: QWidget      ( in_parent )
, splitter_    ( new QSplitter( Qt::Vertical ) )
, out_browser_ ( new QTextBrowser )
, err_browser_ ( new QTextBrowser )
{
   QVBoxLayout* const out_layout = new QVBoxLayout;
   out_layout->addWidget( new QLabel( tr( "Output:" ) ) );
   out_layout->addWidget( out_browser_ );
   out_layout->setContentsMargins( 2, 2, 0, 0 );
   QWidget* const out_panel = new QWidget;
   out_panel->setLayout( out_layout );

   QVBoxLayout* const err_layout = new QVBoxLayout;
   err_layout->addWidget( new QLabel( tr( "Errors:" ) ) );
   err_layout->addWidget( err_browser_ );
   err_layout->setContentsMargins( 2, 2, 0, 0 );
   QWidget* const err_panel = new QWidget;
   err_panel->setLayout( err_layout );
  
   splitter_->addWidget( out_panel );
   splitter_->addWidget( err_panel );

   QVBoxLayout* const main_layout = new QVBoxLayout;
   main_layout->addWidget( splitter_ );
   main_layout->setContentsMargins( 4, 4, 4, 4 );
   setLayout( main_layout );

   setFocusPolicy( Qt::NoFocus );

   QBtEventsController* const evc = QBtEventsController::instance();
   evc->append( this, QBtEvent::OutputMessage );
   evc->append( this, QBtEvent::ErrorMessage );
   evc->append( this, QBtEvent::ClearMessages );
}
// end of QBtOutputWidget

//*******************************************************************
// ~QBtOutputWidget                                       DESTRUCTOR
//*******************************************************************
QBtOutputWidget::~QBtOutputWidget()
{
   QBtEventsController::instance()->remove( this );
}
// end of ~QBtOutputWidget

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtOutputWidget::customEvent( QEvent* const e )
{
   QBtEvent* const event = dynamic_cast<QBtEvent*>( e );
   const int type = static_cast<int>( event->type() );

   switch( type ) {
      case QBtEvent::OutputMessage:
         out_browser_->append( event->data( 0 ).toString() );
         break;
      case QBtEvent::ErrorMessage:
         err_browser_->append( event->data( 0 ).toString() );
         break;
      case QBtEvent::ClearMessages:
         out_browser_->clear();
         err_browser_->clear();
         break;
   }
}
// end of customEvent
