/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtProcess.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 5.02.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtProcess.h"
#include "QBtEventsController.h"
#include <QtDebug>

//*******************************************************************
// QBtProcess                                            CONSTRUCTOR
//*******************************************************************
QBtProcess::QBtProcess( QObject* const in_parent )
: QProcess  ( in_parent )
, out_text_ ( QString() )
, err_text_ ( QString() )
, valid_    ( false     )
{
   connect( this, SIGNAL( readyReadStandardOutput() ), this, SLOT( read_output() ) );
   connect( this, SIGNAL( readyReadStandardError () ), this, SLOT( read_error () ) );
   connect( this, SIGNAL( error(QProcess::ProcessError) ), this, SLOT( error(QProcess::ProcessError) ) );
   connect( this, SIGNAL( finished(int,QProcess::ExitStatus) ), this, SLOT( finished(int,QProcess::ExitStatus) ) );
}
// end of QBtProcess

//*******************************************************************
// run                                                        PUBLIC
//*******************************************************************
void QBtProcess::run( const QString& in_cmd )
{
   out_text_ = err_text_ = QString();
   valid_ = false;
   start( in_cmd );
}
// end of run

//*******************************************************************
// read_output                                          PRIVATE slot
//*******************************************************************
void QBtProcess::read_output()
{
   out_text_ += readAllStandardOutput();
}
// end of read_output

//*******************************************************************
// read_error                                           PRIVATE slot
//*******************************************************************
void QBtProcess::read_error()
{
   err_text_ += readAllStandardError();
}
// end of read_error

//*******************************************************************
// error                                                PRIVATE slot
//*******************************************************************
void QBtProcess::error( const QProcess::ProcessError in_error )
{
   qDebug() << "QBtProcess::error: " << errorString();
   qDebug() << "error: " << in_error;
   emit finished();
}
// end of error

//*******************************************************************
// finished                                             PRIVATE slot
//*******************************************************************
void QBtProcess::finished( const int in_exit_code, const QProcess::ExitStatus in_exit_status )
{
   if( in_exit_code && ( NormalExit != in_exit_status ) ) {
      qDebug() << "QBtProcess::finished: " << errorString();
      qDebug() << "exit code: " << in_exit_code;
      qDebug() << "exit status: " << in_exit_status;
      qDebug() << " ";
   }
   else {
      valid_ = true;
   }
   emit finished();
}
// end of error
