#ifndef INCLUDED_QBtProcessDialog_h
#define INCLUDED_QBtProcessDialog_h
/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtProcessDialog.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 18.04.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtShared.h"
#include <QDialog>
#include <QStringList>

/*------- forward declarations:
-------------------------------------------------------------------*/
class QPushButton;
class QProgressBar;
class QLineEdit;
class QCheckBox;
class QSpinBox;
class QLabel;

/*------- class declaration:
-------------------------------------------------------------------*/
class QBtProcessDialog : public QDialog
{
   Q_OBJECT

//------- CONSTRUCTION/DESTRUCTION -------
public:
   QBtProcessDialog( QWidget*, const Strings&, const QString&, const QStringList& );
   ~QBtProcessDialog();
private:
   QBtProcessDialog( const QBtProcessDialog& );
   QBtProcessDialog& operator=( const QBtProcessDialog& );

//------- CONSTANTS -------
private:
   static const char* const Caption;
   static const char* const Start;
   static const char* const Cancel;
   static const char* const Break;
   static const char* const Progress;
   static const char* const Source;
   static const char* const Destination;
   static const char* const Rename;
   static const char* const RenameInfo;
   static const char* const StartNumber;
   static const QString FileTag;
   static const QString NumberTag;
   //static const char* const Operation;

//------- MEMBERS -------
private:
   Strings             files_;
   QString             dst_dir_;
   QStringList         commands_;
   QPushButton * const start_;
   QPushButton * const cancel_;
   QProgressBar* const progress_;
   QLineEdit   * const source_;
   QLineEdit   * const destination_;
   QCheckBox   * const rename_cbox_;
   QLineEdit   * const rename_muster_;
   QLabel      * const number_label_; 
   QSpinBox    * const number_spin_box_;
   bool                continue_;
   int                 n_;

//------- METHODS -------
private:
   void showEvent( QShowEvent* );
   void customEvent( QEvent* );
   private:
   void run();
   void update_src_info( const QString& );
   void update_dst_info( const QString& );
   void update_operation_info();
   QString dst_fpath( const QString&, const QString&, const QString&, const QStringList&, int& );
   bool rename_muster_valid() const;
   QStringList tags( const QString& ) const;
private slots:
   void start_clicked();
   void cancel_clicked();
   void rename_checked( int );
};

 #endif // INCLUDED_QBtProcessDialog_h
 