/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtShared.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 2.01.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtShared.h"
#include <QDir>
#include <QCursor>
#include <QWidget>
#include <QFileInfo>
#include <QApplication>


/*------- constants:
--------------------------------------------------------------------*/
const char* const QBtShared::Organisation = "beesoft.org";
const char* const QBtShared::ProgramName  = QT_TR_NOOP( "Image Commander" );
const char* const QBtShared::Version      = QT_TR_NOOP( "ver. " );     
const char* const QBtShared::Vernum       = QT_TR_NOOP( "1.3" );

//*******************************************************************
// program_name                                        PUBLIC static
//*******************************************************************
QString QBtShared::program_name()
{
   QString text = tr( ProgramName );
   text += ", ";
   text += tr( Version );
   text += " ";
   text += tr( Vernum );
   return text;
}
// end of program_name

//*******************************************************************
// set_cursor                                          PUBLIC static
//*******************************************************************
void QBtShared::set_cursor( const QCursor& in_cursor )
{
   if( !qApp->overrideCursor() || ( qApp->overrideCursor()->shape() != in_cursor.shape() ) ) {
      qApp->setOverrideCursor( in_cursor );
   }
}
// end of set_cursor

//*******************************************************************
// restore_cursor                                      PUBLIC static
//*******************************************************************
void QBtShared::restore_cursor()
{
   qApp->restoreOverrideCursor();
}
// end of restore_cursor

//*******************************************************************
// disable_updates                                           PRIVATE
//*******************************************************************
void QBtShared::disable_updates( QWidget* const in_widget )
{
   set_cursor( Qt::WaitCursor );
   in_widget->setUpdatesEnabled( false );
   qApp->processEvents();
}
// end of disable_updates

//*******************************************************************
// enable_updates                                            PRIVATE
//*******************************************************************
void QBtShared::enable_updates( QWidget* const in_widget )
{
   in_widget->setUpdatesEnabled( true );
   restore_cursor();
   qApp->processEvents();
}
// end of enable_updates

//*******************************************************************
// reverse
//*******************************************************************
QString& QBtShared::reverse( QString& in_text )
{
   const int n = in_text.length();
   if( n ) {
      int idx1 = 0;
      int idx2 = n - 1;
      while( idx1 < idx2 ) {
         const QChar tmp = in_text[ idx1 ];
         in_text[ idx1 ] = in_text[ idx2 ];
         in_text[ idx2 ] = tmp;
         ++idx1;
         --idx2;
      }
   }
   return in_text;
}
// end of reverse

//*******************************************************************
// num2str
//*******************************************************************
QString QBtShared::num2str( const qint64 in_value )
{
   static const QChar Space = ' ';
   
   QString   text = QString::number( in_value );
   const int n = text.length();
   if( n < 4 ) return text;
   
   reverse( text );
   
   const int n_div_3 = n / 3;
   const int m       = ( n % 3 )  ? n_div_3 : ( n_div_3 - 1 );
   QString result( n + m, Space );

   int src_idx = 0;
   int dst_idx = 0;
   int k       = 0;

   while( src_idx < n ) {
      if( 3 == k ) {
         result[ dst_idx++ ] = ',';
         k = 0;
      }
      result[ dst_idx++ ]= text[ src_idx++ ];
      ++k;
   }   
   
   return reverse( result );
}
// end of num2str

//*******************************************************************
// dst_fpath
//*******************************************************************
QString QBtShared::dst_fpath( const QString& src_fpath, const QString& dst_dir )
{
   const QFileInfo info( src_fpath );

   QString dst_fpath = dst_dir;
   if( !dst_fpath.endsWith( QDir::separator() ) ) dst_fpath += QDir::separator();
   dst_fpath += info.completeBaseName();
   dst_fpath += '.';
   dst_fpath += info.suffix();

   return dst_fpath;
}
// end of dst_fpath
