#ifndef INCLUDED_QBtView_h
#define INCLUDED_QBtView_h
/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtView.h
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 19.01.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtShared.h"
#include <QTreeWidget>
#include <QDir>
#include <set>
#include <stack>
#include <vector>

/*------- forward declarations:
-------------------------------------------------------------------*/
class QBtCmdWidget;

/*------- class declaration:
-------------------------------------------------------------------*/
class QBtView : public QTreeWidget
{
   Q_OBJECT

//------- TYPES -------
private:
   enum {
      FLD_CHECK_INDEX = 0,
      FLD_NAME_INDEX,
      FLD_EXT_INDEX,
      FLD_ACCESS_INDEX,
      FLD_SIZE_INDEX,
      FLD_DATE_INDEX,
      FLD_TIME_INDEX,

      IS_DIR = Qt::UserRole,
      IS_PIC,
      PATH,
   };

   typedef std::stack<QString> NamesStack;
   typedef std::set<QString> Extensions;
   typedef std::vector<QTreeWidgetItem*> SelectedItems;

//------- CONSTRUCTION / DESTRUCTION -------
public:
   QBtView( const QString&, QWidget* );
   ~QBtView();
private:
   QBtView( const QBtView& );
   QBtView& operator=( const QBtView& );

//------- CONSTANTS -------
private:
   static const char* const Name;
   static const char* const Ext;
   static const char* const Access;
   static const char* const Size;
   static const char* const Date;
   static const char* const Time;
   static const char* const MkDirCaption;
   static const char* const MkDirPrompt;
   static const char* const Error;
   static const char* const DirNotEmpty;

//------- MEMBERS -------
protected:
   QString path_;
private:
   QDir dir_;
   NamesStack names_;
   Extensions exts_;
   
//------- METHODS -------
public:
   void             viewer         ();
   Strings          get_selected   ();
   QString          get_current    () const;
   const QString&   path           () const { return path_; }
   void             reread         ();
   void             delete_file     ( const QString& );
   void             current_file    ( const QString& );
   void             check_file      ( const QString& );
protected:
   bool             read            ( const QString&   );
private:
   StringSet        get_selected_set();
   void             focusInEvent    ( QFocusEvent* );
   void             customEvent     ( QEvent* );
   void             read            ();
   void             drawRow         ( QPainter*, const QStyleOptionViewItem&, const QModelIndex& ) const;
   void             keyPressEvent   ( QKeyEvent* );
   void             add_dot_dot     ();
   void             add_item        ( const QFileInfo& );
   void             resize_columns  ();
   QString          access          ( const QFileInfo& ) const;
   void             goto_top        ();
   void             goto_item       ( QTreeWidgetItem* );
   void             goto_end        ();
   void             cd_up           ();
   QTreeWidgetItem* get_next_item   ( const QTreeWidgetItem* ) const;
   QTreeWidgetItem* search_file     ( const QString& );
   QTreeWidgetItem* search_item     ( const QString& );
   QTreeWidgetItem* search_item     ( const QString&, const QString& );
   SelectedItems    selected_items  ();
   void             select_pictures ();
   QString          get_full_name   ( const QTreeWidgetItem* ) const;
   Strings          search_pics     ();
   void             disable_updates ();
   void             enable_updates  ();
   void             make_dir        ();
   void             remove          ();
   void             remove_item     ( QTreeWidgetItem* );
   void             info            ();
   void             grayscale       ();
   void             negate          ();
   void             normalize       ();
   void             enhance         ();
   void             equalize        ();
   void             watermark       ();
   void             rotate          ();
   void             resize          ();
   void             border          ();
   void             annotate        ();
private slots:
   void             execute         ();
};

#endif // INCLUDED_QBtView_h
