/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtViewWidget.cpp
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 8.01.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtViewWidget.h"
#include "QBtViewLeft.h"
#include "QBtViewRight.h"
#include "QBtEventsController.h"
#include "QBtButtons.h"
#include <QSplitter>
#include <QVBoxLayout>

//*******************************************************************
// QBtViewWidget                                          CONSTRUCTOR
//*******************************************************************
QBtViewWidget::QBtViewWidget( QWidget* const in_parent )
: QWidget   ( in_parent        )
, splitter_ ( new QSplitter    )
, viewA_    ( new QBtViewLeft  )
, viewB_    ( new QBtViewRight )
{
   splitter_->addWidget( viewA_ );
   splitter_->addWidget( viewB_ );
   
   QVBoxLayout* const main_layout = new QVBoxLayout;
   main_layout->addWidget( splitter_ );
   main_layout->addWidget( new QBtButtons );
   main_layout->setStretchFactor( splitter_, 100 );
   main_layout->setSpacing( 1 );
   main_layout->setContentsMargins( 4, 4, 4, 0 );
   setLayout( main_layout );

   setFocusPolicy( Qt::NoFocus );

   QBtEventsController* const evc = QBtEventsController::instance();
   evc->append( this, QBtEvent::SwitchPanel );
   evc->append( this, QBtEvent::F3 );
}
// end of QBtViewWidget

//*******************************************************************
// ~QBtWorkspace                                          DESTRUCTOR
//*******************************************************************
QBtViewWidget::~QBtViewWidget()
{
   QBtEventsController::instance()->remove( this );
}
// end of ~QBtWorkspace

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtViewWidget::customEvent( QEvent* const in_event )
{
   QBtEvent* const event = dynamic_cast<QBtEvent*>( in_event );
   const int type = static_cast<int>( event->type() );

   switch( type ) {
      case QBtEvent::SwitchPanel:
         switch_panel();
         break;
      case QBtEvent::F3:
         viewer();
         break;
   }
}
// end of customEvent

//*******************************************************************
// switch_panel                                              PRIVATE
//*******************************************************************
void QBtViewWidget::switch_panel()
{
   if( !viewA_->hasFocus() ) viewA_->setFocus( Qt::TabFocusReason );
   else                      viewB_->setFocus( Qt::TabFocusReason );
}
// end of switch_panel

//*******************************************************************
// viewer                                                    PRIVATE
//*******************************************************************
void QBtViewWidget::viewer()
{
        if( viewA_->hasFocus() ) viewA_->viewer();
   else if( viewB_->hasFocus() ) viewB_->viewer();
}
// end of viewer

