#ifndef INCLUDED_QBtViewer_h
#define INCLUDED_QBtViewer_h
/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Image Commander
 *
 * Image Commander is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Image Commander is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Image Commander; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * Project      : Image Commander
 * File         : QBtViewer.h
 * Author       : Piotr Pszczolkowski
 * Contact      : piotr@beesoft.pl
 * Creation date: 29.01.2009
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtShared.h"
#include <QDialog>
#include <QImage>

/*-------- forward declarations:
-------------------------------------------------------------------*/
class QPushButton;
class QCheckBox;
class QBtPicture;

/*------- class declaration:
-------------------------------------------------------------------*/
class QBtViewer : public QDialog
{
   Q_OBJECT

//------- CONSTRUCTON / DESTRUCTION -------
public:
   QBtViewer( QWidget*, const Strings&, int, const StringSet& );
   QBtViewer( QWidget*, const QImage& );
private:
   QBtViewer( const QBtViewer& );
   QBtViewer& operator=( const QBtViewer& );

//------- CONSTANTS -------
private:
   static const char* const Caption;
   static const char* const Save;
   static const char* const Close;
   static const char* const Scale;
   static const char* const Error;
   static const char* const Previous;
   static const char* const Next;
   static const char* const RotateLeft;
   static const char* const RotateRight;
   static const char* const Check;
   static const char* const Delete;
   static const char* const DeleteError;

//------- MEMBERS -------
private:
   Strings            files_;
   unsigned int       index_;
   StringSet          checked_;
   QWidget*           panel_;
   QBtPicture * const picture_;
   QCheckBox  * const scale_;
   QPushButton* const left_;
   QPushButton* const right_;
   QPushButton* const prev_;
   QPushButton* const next_;
   QPushButton* const check_;
   QPushButton* const delete_;
   QPushButton* const save_;
   QPushButton* const close_;
   QImage             image_;
   int                angle_;
   bool               ok_;
   bool               scaled_;

//------- METHODS -------
private:
   void showEvent     ( QShowEvent* );
   void resizeEvent   ( QResizeEvent* );
   void resize_image  ( bool = true );
   void display       ();
   void check_buttons ();
private slots:
   void scale_changed ( int );
   void next          ();
   void prev          ();
   void left          ();
   void right         ();
   void save_clicked  ();
   void check_clicked ();
   void delete_clicked();
};

#endif // INCLUDED_QBtViewer_h
